@@ -16,7 +16,8 @@ import numpy as np
 from ..extern import six
 from . import angle_utilities as util
 from .. import units as u
-from ..utils import deprecated, isiterable
+from ..utils import isiterable
+from ..utils.compat import NUMPY_LT_1_7
 
 
 __all__ = ['Angle', 'Latitude', 'Longitude']
@@ -48,6 +49,8 @@ class Angle(u.Quantity):
       Angle(u'1°2′3″')
       Angle('1d2m3.4s')
       Angle('-1h2m3s')
+      Angle('-1h2.5m')
+      Angle('-1:2.5', unit=u.deg)
       Angle((-1, 2, 3), unit=u.deg)  # (d, m, s)
       Angle(10.2345 * u.deg)
       Angle(Angle(10.2345 * u.deg))
@@ -137,13 +140,12 @@ class Angle(u.Quantity):
     @staticmethod
     def _tuple_to_float(angle, unit):
         """
-        Converts an angle represented as a 3-tuple into a floating
+        Converts an angle represented as a 3-tuple or 2-tuple into a floating
         point number in the given unit.
         """
         if isinstance(angle, tuple):
             # TODO: Numpy array of tuples?
             if unit is u.hourangle:
-                util.check_hms_ranges(*angle)
                 angle = util.hms_to_hours(*angle)
             elif unit is u.degree:
                 angle = util.dms_to_degrees(*angle)
@@ -199,7 +201,7 @@ class Angle(u.Quantity):
         members.  The ``d``, ``m``, ``s`` are thus always positive, and the sign of
         the angle is given by ``sign``. (This is a read-only property.)
 
-        This is primarily intented for use with `dms` to generate string
+        This is primarily intended for use with `dms` to generate string
         representations of coordinates that are correct for negative angles.
         """
         return signed_dms_tuple(np.sign(self.degree),
@@ -218,7 +220,7 @@ class Angle(u.Quantity):
             used.
 
         decimal : bool, optional
-            If `True`, a decimal respresentation will be used, otherwise
+            If `True`, a decimal representation will be used, otherwise
             the returned string will be in sexagesimal form.
 
         sep : str, optional
@@ -267,8 +269,10 @@ class Angle(u.Quantity):
 
         Returns
         -------
-        strrepr : str
-            A string representation of the angle.
+        strrepr : str or array
+            A string representation of the angle. If the angle is an array, this
+            will be an array with a unicode dtype.
+
 
         """
         if unit is None:
@@ -366,8 +370,19 @@ class Angle(u.Quantity):
                 s = '${0}$'.format(s)
             return s
 
-        format_ufunc = np.vectorize(do_format, otypes=[np.object])
-        result = format_ufunc(values)
+        if NUMPY_LT_1_7 and not np.isscalar(values):  # pragma: no cover
+            format_ufunc = np.vectorize(do_format, otypes=[np.object])
+            # In Numpy 1.6, unicode output is broken.  vectorize always seems to
+            # yieled U2 even if you tell it something else.  So we convert in
+            # a second step with 60 chars, on the theory that you'll never want
+            # better than what double-precision decimals give, which end up
+            # around that many characters.
+            result = format_ufunc(values).astype('U60')
+        else:
+            #for newer Numpy versions, this just works as you would expect
+            format_ufunc = np.vectorize(do_format, otypes=['U'])
+            result = format_ufunc(values)
+
         if result.ndim == 0:
             result = result[()]
         return result
@@ -461,18 +476,22 @@ class Angle(u.Quantity):
             ok &= np.all(self < Angle(upper))
         return bool(ok)
 
-    @deprecated("0.3", name="format", alternative="to_string")
-    def format(self, unit=u.degree, decimal=False, sep='fromunit', precision=5,
-               alwayssign=False, pad=False):
-        return self.to_string(
-            unit=unit, decimal=decimal, sep=sep, precision=precision,
-            alwayssign=alwayssign, pad=pad)
-
     def __str__(self):
         return str(self.to_string())
 
     def _repr_latex_(self):
-        return str(self.to_string(format='latex'))
+        if self.isscalar:
+            return self.to_string(format='latex')
+        else:
+            # Need to do a magic incantation to convert to str.  Regular str
+            # or array2string causes all backslashes to get doubled.
+            if NUMPY_LT_1_7:
+                # Except that numpy 1.6 doesn't do formatter... so instead we
+                # just replace all double-backslashes with one.
+                return str(self.to_string(format='latex')).replace('\\\\', '\\')
+            else:
+                return np.array2string(self.to_string(format='latex'),
+                                       formatter={'str_kind': lambda x: x})
 
 
 class Latitude(Angle):
@@ -531,7 +550,7 @@ class Latitude(Angle):
 
     def _validate_angles(self, angles=None):
         """Check that angles are between -90 and 90 degrees.
-        If not given, the check is done on the object iself"""
+        If not given, the check is done on the object itself"""
         # Convert the lower and upper bounds to the "native" unit of
         # this angle.  This limits multiplication to two values,
         # rather than the N values in `self.value`.  Also, the
@@ -666,7 +685,7 @@ class Longitude(Angle):
 
     def __array_finalize__(self, obj):
         super(Longitude, self).__array_finalize__(obj)
-        self._wrap_angle = getattr(obj, '_wrap_angle', None)
+        self._wrap_angle = getattr(obj, '_wrap_angle', 360 * u.deg)
 
     # Any calculation should drop to Angle
     def __array_wrap__(self, obj, context=None):
