@@ -15,6 +15,7 @@ from ..angles import Longitude, Latitude, Angle
 from ...tests.helper import pytest
 from ... import units as u
 from ..errors import IllegalSecondError, IllegalMinuteError, IllegalHourError
+from ...utils.compat import NUMPY_LT_1_7
 
 
 def test_create_angles():
@@ -57,24 +58,37 @@ def test_create_angles():
     with pytest.raises(ValueError):
         a13 = Angle(12.34, unit="not a unit")
 
-    a14 = Angle("12h43m32") # no trailing 's', but unambiguous
+    a14 = Angle("03h36m29.7888000120") # no trailing 's', but unambiguous
 
     a15 = Angle("5h4m3s") # single digits, no decimal
 
     a16 = Angle("1 d")
     a17 = Angle("1 degree")
+
     assert a16.degree == 1
     assert a17.degree == 1
 
+    a18 = Angle("54 07.4472", unit=u.degree)
+    a19 = Angle("54:07.4472", unit=u.degree)
+    a20 = Angle("54d07.4472m", unit=u.degree)
+    a21 = Angle("3h36m", unit=u.hour)
+    a22 = Angle("3.6h", unit=u.hour)
+    a23 = Angle("- 3h", unit=u.hour)
+    a24 = Angle("+ 3h", unit=u.hour)
+
     #ensure the above angles that should match do
-    assert a1 == a2 == a3 == a4 == a5 == a6 == a7
+    assert a1 == a2 == a3 == a4 == a5 == a6 == a7 == a8 == a18 == a19 == a20
     assert_allclose(a1.radian, a2.radian)
     assert_allclose(a2.degree, a3.degree)
     assert_allclose(a3.radian, a4.radian)
     assert_allclose(a4.radian, a5.radian)
     assert_allclose(a5.radian, a6.radian)
     assert_allclose(a6.radian, a7.radian)
-    #assert a10 == a11 == a12
+
+    assert_allclose(a10.degree, a11.degree)
+    assert a11 == a12 == a14
+    assert a21 == a22
+    assert a23 == -a24
 
     # check for illegal ranges / values
     with pytest.raises(IllegalSecondError):
@@ -108,7 +122,8 @@ def test_angle_ops():
     # Angles can be added and subtracted. Multiplication and division by a
     # scalar is also permitted. A negative operator is also valid.  All of
     # these operate in a single dimension. Attempting to multiply or divide two
-    # Angle objects will raise an exception.
+    # Angle objects will return a quantity.  An exception will be raised if it
+    # is attempted to store output with a non-angular unit in an Angle [#2718].
 
     a1 = Angle(3.60827466667, unit=u.hour)
     a2 = Angle("54:07:26.832", unit=u.degree)
@@ -135,6 +150,16 @@ def test_angle_ops():
     assert a1 < a5
     assert a1 <= a5
 
+    a6 = Angle(45., u.degree)
+    a7 = a6 * a5
+    assert type(a7) is u.Quantity
+
+    with pytest.raises(TypeError):
+        a6 *= a5
+
+    with pytest.raises(TypeError):
+        np.sin(a6, out=a6)
+
 
 def test_angle_convert():
     """
@@ -271,7 +296,6 @@ def test_angle_formatting():
     res = 'Angle as rad decimal: 0.0629763'
     assert "Angle as rad decimal: {0}".format(angle.to_string(unit=u.radian, decimal=True)) == res
 
-
     # check negative angles
 
     angle = Angle(-1.23456789, unit=u.degree)
@@ -283,6 +307,14 @@ def test_angle_formatting():
     assert angle2.to_string(unit=u.hour, pad=True) == '-01h14m04.4444s'
     assert angle.to_string(unit=u.radian, decimal=True) == '-0.0215473'
 
+
+def test_to_string_vector():
+    # Regression test for the fact that vectorize doesn't work with Numpy 1.6
+    assert Angle([1./7., 1./7.], unit='deg').to_string()[0] == "0d08m34.2857s"
+    assert Angle([1./7.], unit='deg').to_string()[0] == "0d08m34.2857s"
+    assert Angle(1./7., unit='deg').to_string() == "0d08m34.2857s"
+
+
 def test_angle_format_roundtripping():
     """
     Ensures that the string represtation of an angle can be used to create a
@@ -804,3 +836,50 @@ def test_angle_axis():
 
     assert an2 - 89*u.deg < 1e-10*u.deg
     assert_allclose(ax2, [-2**-0.5, -2**-0.5, 0])
+
+def test_array_angle_tostring():
+    aobj = Angle([1, 2], u.deg)
+    assert aobj.to_string().dtype.kind == 'U'
+    assert np.all(aobj.to_string() == ['1d00m00s', '2d00m00s'])
+
+def test_wrap_at_without_new():
+    """
+    Regression test for subtle bugs from situations where an Angle is
+    created via numpy channels that don't do the standard __new__ but instead
+    depend on array_finalize to set state.  Longitude is used because the
+    bug was in its _wrap_angle not getting initialized correctly
+    """
+    l1 = Longitude([1]*u.deg)
+    l2 = Longitude([2]*u.deg)
+
+    l = np.concatenate([l1, l2])
+    assert l._wrap_angle is not None
+
+
+def test_repr_latex():
+    """
+    Check the _repr_latex_ method, used primarily by IPython notebooks
+    """
+
+    # try with both scalar
+    scangle = Angle(2.1, u.deg)
+    rlscangle = scangle._repr_latex_()
+
+    # and array angles
+    arrangle = Angle([1, 2.1], u.deg)
+    rlarrangle = arrangle._repr_latex_()
+
+    if NUMPY_LT_1_7:
+        # numpy 1.6 gives weird latex output for unclear reasons.  We don't care
+        # that much though, so just xfail after making sure it isn't
+        # over-backslashing
+        assert '\\\\' not in rlscangle
+        assert '\\\\' not in rlarrangle
+        pytest.xfail('numpy 1.6 does not give correct to_string latex output')
+
+    assert rlscangle == '$2^\circ06{}^\prime00{}^{\prime\prime}$'
+    assert rlscangle.split('$')[1] in rlarrangle
+
+    # make sure the ... appears for large arrays
+    bigarrangle = Angle(np.ones(50000)/50000., u.deg)
+    assert '...' in bigarrangle._repr_latex_()
