@@ -5,8 +5,10 @@ try:
 except ImportError: # cStringIO doesn't exist in Python 3
     from io import BytesIO
     StringIO = lambda x: BytesIO(x.encode('ascii'))
-from tempfile import NamedTemporaryFile
 import os
+import functools
+
+from textwrap import dedent
 
 import numpy as np
 from numpy import ma
@@ -22,7 +24,6 @@ from ....tests.helper import pytest
 from ....extern import six
 
 TRAVIS = os.environ.get('TRAVIS', False)
-pytestmark = pytest.mark.skipif(os.environ.get('APPVEYOR'),  reason="fails on AppVeyor")
 
 def assert_table_equal(t1, t2, check_meta=False):
     assert_equal(len(t1), len(t2))
@@ -44,8 +45,14 @@ def assert_table_equal(t1, t2, check_meta=False):
                 except (TypeError, NotImplementedError):
                     pass # ignore for now
 
-def _read(table, Reader, format, parallel=False, check_meta=False, **kwargs):
+# Use this counter to create a unique filename for each file created in a test
+# if this function is called more than once in a single test
+_filename_counter = 0
+
+def _read(tmpdir, table, Reader=None, format=None, parallel=False, check_meta=False, **kwargs):
     # make sure we have a newline so table can't be misinterpreted as a filename
+    global _filename_counter
+
     table += '\n'
     reader = Reader(**kwargs)
     t1 = reader.read(table)
@@ -67,39 +74,58 @@ def _read(table, Reader, format, parallel=False, check_meta=False, **kwargs):
             'parallel': True}, **kwargs)
         assert_table_equal(t1, t6, check_meta=check_meta)
 
-    with NamedTemporaryFile() as f:
+    filename = str(tmpdir.join('table{0}.txt'.format(_filename_counter)))
+    _filename_counter += 1
+
+    with open(filename, 'wb') as f:
         f.write(table.encode('ascii'))
         f.flush()
-        t7 = ascii.read(f.name, format=format, guess=False, **kwargs)
-        if parallel:
-            t8 = ascii.read(f.name, format=format, guess=False, fast_reader={
-                'parallel': True}, **kwargs)
+
+    t7 = ascii.read(filename, format=format, guess=False, **kwargs)
+    if parallel:
+        t8 = ascii.read(filename, format=format, guess=False, fast_reader={
+            'parallel': True}, **kwargs)
 
     assert_table_equal(t1, t7, check_meta=check_meta)
     if parallel:
         assert_table_equal(t1, t8, check_meta=check_meta)
     return t1
 
-def read_basic(table, **kwargs):
-    return _read(table, FastBasic, 'basic', **kwargs)
 
-def read_csv(table, **kwargs):
-    return _read(table, FastCsv, 'csv', **kwargs)
+@pytest.fixture(scope='function')
+def read_basic(tmpdir, request):
+    return functools.partial(_read, tmpdir, Reader=FastBasic, format='basic')
 
-def read_tab(table, **kwargs):
-    return _read(table, FastTab, 'tab', **kwargs)
 
-def read_commented_header(table, **kwargs):
-    return _read(table, FastCommentedHeader, 'commented_header', **kwargs)
+@pytest.fixture(scope='function')
+def read_csv(tmpdir, request):
+    return functools.partial(_read, tmpdir, Reader=FastCsv, format='csv')
 
-def read_rdb(table, **kwargs):
-    return _read(table, FastRdb, 'rdb', **kwargs)
 
-def read_no_header(table, **kwargs):
-    return _read(table, FastNoHeader, 'no_header', **kwargs)
+@pytest.fixture(scope='function')
+def read_tab(tmpdir, request):
+    return functools.partial(_read, tmpdir, Reader=FastTab, format='tab')
+
+
+@pytest.fixture(scope='function')
+def read_commented_header(tmpdir, request):
+    return functools.partial(_read, tmpdir, Reader=FastCommentedHeader,
+                             format='commented_header')
+
+
+@pytest.fixture(scope='function')
+def read_rdb(tmpdir, request):
+    return functools.partial(_read, tmpdir, Reader=FastRdb, format='rdb')
+
+
+@pytest.fixture(scope='function')
+def read_no_header(tmpdir, request):
+    return functools.partial(_read, tmpdir, Reader=FastNoHeader,
+                             format='no_header')
+
 
 @pytest.mark.parametrize("parallel", [True, False])
-def test_simple_data(parallel):
+def test_simple_data(parallel, read_basic):
     """
     Make sure the fast reader works with basic input data.
     """
@@ -107,6 +133,7 @@ def test_simple_data(parallel):
     expected = Table([[1, 4], [2, 5], [3, 6]], names=('A', 'B', 'C'))
     assert_table_equal(table, expected)
 
+
 def test_read_types():
     """
     Make sure that the read() function takes filenames,
@@ -119,8 +146,9 @@ def test_read_types():
     assert_table_equal(t1, t2)
     assert_table_equal(t2, t3)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_supplied_names(parallel):
+def test_supplied_names(parallel, read_basic):
     """
     If passed as a parameter, names should replace any
     column names found in the header.
@@ -129,8 +157,9 @@ def test_supplied_names(parallel):
     expected = Table([[1, 4], [2, 5], [3, 6]], names=('X', 'Y', 'Z'))
     assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_no_header(parallel):
+def test_no_header(parallel, read_basic, read_no_header):
     """
     The header should not be read when header_start=None. Unless names is
     passed, the column names should be auto-generated.
@@ -141,8 +170,9 @@ def test_no_header(parallel):
     assert_table_equal(t1, expected)
     assert_table_equal(t2, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_no_header_supplied_names(parallel):
+def test_no_header_supplied_names(parallel, read_basic):
     """
     If header_start=None and names is passed as a parameter, header
     data should not be read and names should be used instead.
@@ -152,8 +182,9 @@ def test_no_header_supplied_names(parallel):
     expected = Table([['A', '1', '4'], ['B', '2', '5'], ['C', '3', '6']], names=('X', 'Y', 'Z'))
     assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_comment(parallel):
+def test_comment(parallel, read_basic):
     """
     Make sure that line comments are ignored by the C reader.
     """
@@ -161,8 +192,9 @@ def test_comment(parallel):
     expected = Table([[1, 4], [2, 5], [3, 6]], names=('A', 'B', 'C'))
     assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_empty_lines(parallel):
+def test_empty_lines(parallel, read_basic):
     """
     Make sure that empty lines are ignored by the C reader.
     """
@@ -170,8 +202,9 @@ def test_empty_lines(parallel):
     expected = Table([[1, 4], [2, 5], [3, 6]], names=('A', 'B', 'C'))
     assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_lstrip_whitespace(parallel):
+def test_lstrip_whitespace(parallel, read_basic):
     """
     Test to make sure the reader ignores whitespace at the beginning of fields.
     """
@@ -185,8 +218,9 @@ def test_lstrip_whitespace(parallel):
     expected = Table([['A', 'a'], ['B', 'b'], ['C', 'c']], names=('1', '2', '3'))
     assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_rstrip_whitespace(parallel):
+def test_rstrip_whitespace(parallel, read_basic):
     """
     Test to make sure the reader ignores whitespace at the end of fields.
     """
@@ -195,8 +229,9 @@ def test_rstrip_whitespace(parallel):
     expected = Table([['A', 'a'], ['B', 'b'], ['C', 'c']], names=('1', '2', '3'))
     assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_conversion(parallel):
+def test_conversion(parallel, read_basic):
     """
     The reader should try to convert each column to ints. If this fails, the
     reader should try to convert to floats. Failing this, it should fall back
@@ -215,8 +250,9 @@ A B C D E
     assert_equal(table['D'].dtype.kind, 'f')
     assert table['E'].dtype.kind in ('S', 'U')
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_delimiter(parallel):
+def test_delimiter(parallel, read_basic):
     """
     Make sure that different delimiters work as expected.
     """
@@ -231,8 +267,9 @@ COL1 COL2 COL3
         table = read_basic(text.replace(' ', sep), delimiter=sep, parallel=parallel)
         assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_include_names(parallel):
+def test_include_names(parallel, read_basic):
     """
     If include_names is not None, the parser should read only those columns in include_names.
     """
@@ -240,8 +277,9 @@ def test_include_names(parallel):
     expected = Table([[1, 5], [4, 8]], names=('A', 'D'))
     assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_exclude_names(parallel):
+def test_exclude_names(parallel, read_basic):
     """
     If exclude_names is not None, the parser should exclude the columns in exclude_names.
     """
@@ -249,8 +287,9 @@ def test_exclude_names(parallel):
     expected = Table([[2, 6], [3, 7]], names=('B', 'C'))
     assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_include_exclude_names(parallel):
+def test_include_exclude_names(parallel, read_basic):
     """
     Make sure that include_names is applied before exclude_names if both are specified.
     """
@@ -264,8 +303,9 @@ A B C D E F G H
     expected = Table([[1, 9], [4, 12], [8, 16]], names=('A', 'D', 'H'))
     assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_quoted_fields(parallel):
+def test_quoted_fields(parallel, read_basic):
     """
     The character quotechar (default '"') should denote the start of a field which can
     contain the field delimiter and newlines.
@@ -284,6 +324,7 @@ a b "   c
     table = read_basic(text.replace('"', "'"), quotechar="'", parallel=parallel)
     assert_table_equal(table, expected)
 
+
 def test_invalid_parameters():
     """
     Make sure the C reader raises an error if passed parameters it can't handle.
@@ -317,6 +358,7 @@ def test_invalid_parameters():
         # Outputter cannot be specified in constructor
         table = FastBasic(Outputter=ascii.TableOutputter).read('1 2 3\n4 5 6')
 
+
 def test_too_many_cols():
     """
     If a row contains too many columns, the C reader should raise an error.
@@ -333,8 +375,9 @@ A B C
     assert 'CParserError: an error occurred while parsing table data: too many ' \
         'columns found in line 3 of data' in str(e)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_not_enough_cols(parallel):
+def test_not_enough_cols(parallel, read_csv):
     """
     If a row does not have enough columns, the FastCsv reader should add empty
     fields while the FastBasic reader should raise an error.
@@ -352,8 +395,9 @@ A,B,C
     with pytest.raises(CParserError) as e:
         table = FastBasic(delimiter=',').read(text)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_data_end(parallel):
+def test_data_end(parallel, read_basic, read_rdb):
     """
     The parameter data_end should specify where data reading ends.
     """
@@ -394,8 +438,39 @@ N\tN\tS
     expected = Table([[], [], []], names=('A', 'B', 'C'))
     assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_fill_values(parallel):
+def test_inf_nan(parallel, read_basic):
+    """
+    Test that inf and nan-like values are correctly parsed on all platforms.
+
+    Regression test for https://github.com/astropy/astropy/pull/3525
+    """
+
+    text = dedent("""\
+        A
+        nan
+        +nan
+        -nan
+        inf
+        infinity
+        +inf
+        +infinity
+        -inf
+        -infinity
+    """)
+
+    expected = Table({'A': [np.nan, np.nan, np.nan,
+                            np.inf, np.inf, np.inf, np.inf,
+                            -np.inf, -np.inf]})
+
+    table = read_basic(text, parallel=parallel)
+    assert table['A'].dtype.kind == 'f'
+    assert_table_equal(table, expected)
+
+
+@pytest.mark.parametrize("parallel", [True, False])
+def test_fill_values(parallel, read_basic):
     """
     Make sure that the parameter fill_values works as intended. If fill_values
     is not specified, the default behavior should be to convert '' to 0.
@@ -441,8 +516,9 @@ nan, 5, -9999
     assert_almost_equal(table['C'].data.data[0], 999.0)
     assert_almost_equal(table['C'][1], -3.4) # column is still of type float
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_fill_include_exclude_names(parallel):
+def test_fill_include_exclude_names(parallel, read_csv):
     """
     fill_include_names and fill_exclude_names should filter missing/empty value handling
     in the same way that include_names and exclude_names filter output columns.
@@ -468,8 +544,9 @@ A, B, C
     assert table['B'][1] is not ma.masked # fill_exclude_names applies after fill_include_names
     assert table['C'][2] is not ma.masked
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_many_rows(parallel):
+def test_many_rows(parallel, read_basic):
     """
     Make sure memory reallocation works okay when the number of rows
     is large (so that each column string is longer than INITIAL_COL_SIZE).
@@ -483,8 +560,9 @@ def test_many_rows(parallel):
     expected = Table([[0] * 500, [1] * 500, [2] * 500], names=('A', 'B', 'C'))
     assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_many_columns(parallel):
+def test_many_columns(parallel, read_basic):
     """
     Make sure memory reallocation works okay when the number of columns
     is large (so that each hedaer string is longer than INITIAL_HEADER_SIZE).
@@ -496,6 +574,7 @@ def test_many_columns(parallel):
     expected = Table([[i, i] for i in range(500)], names=[str(i) for i in range(500)])
     assert_table_equal(table, expected)
 
+
 def test_fast_reader():
     """
     Make sure that ascii.read() works as expected by default and with
@@ -506,8 +585,18 @@ def test_fast_reader():
         ascii.read(text, format='fast_basic', guess=False, comment='##')
 
     # Enable multiprocessing and the fast converter
-    ascii.read(text, format='basic', guess=False, fast_reader={'parallel': True,
-                                                    'use_fast_converter': True})
+    try:
+        ascii.read(text, format='basic', guess=False,
+                   fast_reader={'parallel': True, 'use_fast_converter': True})
+    except NotImplementedError:
+        # Might get this on Windows, try without parallel...
+        if os.name == 'nt':
+            ascii.read(text, format='basic', guess=False,
+                       fast_reader={'parallel': False,
+                                    'use_fast_converter': True})
+        else:
+            raise
+
     # Should raise an error if fast_reader has an invalid key
     with pytest.raises(FastOptionsError):
         ascii.read(text, format='fast_basic', guess=False, fast_reader={'foo': True})
@@ -517,8 +606,9 @@ def test_fast_reader():
     # Will try the slow reader afterwards by default
     ascii.read(text, format='basic', guess=False, comment='##')
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_read_tab(parallel):
+def test_read_tab(parallel, read_tab):
     """
     The fast reader for tab-separated values should not strip whitespace, unlike
     the basic reader.
@@ -533,8 +623,9 @@ def test_read_tab(parallel):
     assert_equal(table['2'][1], ' d e')  # preserve whitespace in quoted fields
     assert_equal(table['3'][1], '  ')    # preserve end-of-line whitespace
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_default_data_start(parallel):
+def test_default_data_start(parallel, read_basic):
     """
     If data_start is not explicitly passed to read(), data processing should
     beginning right after the header.
@@ -544,8 +635,9 @@ def test_default_data_start(parallel):
     expected = Table([[1, 4], [2, 5], [3, 6]], names=('a', 'b', 'c'))
     assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_commented_header(parallel):
+def test_commented_header(parallel, read_commented_header):
     """
     The FastCommentedHeader reader should mimic the behavior of the
     CommentedHeader by overriding the default header behavior of FastBasic.
@@ -573,8 +665,9 @@ def test_commented_header(parallel):
     with pytest.raises(ParameterError):
         read_commented_header(text, header_start=-1, data_start=-1, parallel=parallel) # data_start cannot be negative
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_rdb(parallel):
+def test_rdb(parallel, read_rdb):
     """
     Make sure the FastRdb reader works as expected.
     """
@@ -606,8 +699,9 @@ A\tB\tC
         read_rdb(text, parallel=parallel)
     assert 'type definitions do not all match [num](N|S)' in str(e)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_data_start(parallel):
+def test_data_start(parallel, read_basic):
     """
     Make sure that data parsing begins at data_start (ignoring empty and
     commented lines but not taking quoted values into account).
@@ -657,8 +751,9 @@ A B C
     expected = Table([[4, 7, 10], [5, 8, 11], [6, 9, 12]], names=('A', 'B', 'C'))
     assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_quoted_empty_values(parallel):
+def test_quoted_empty_values(parallel, read_basic):
     """
     Quoted empty values spanning multiple lines should be treated correctly.
     """
@@ -668,8 +763,9 @@ def test_quoted_empty_values(parallel):
     table = read_basic(text, parallel=parallel)
     assert table['c'][0] is ma.masked # empty value masked by default
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_csv_comment_default(parallel):
+def test_csv_comment_default(parallel, read_csv):
     """
     Unless the comment parameter is specified, the CSV reader should
     not treat any lines as comments.
@@ -679,8 +775,9 @@ def test_csv_comment_default(parallel):
     expected = Table([['#1', '4'], [2, 5], [3, 6]], names=('a', 'b', 'c'))
     assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_whitespace_before_comment(parallel):
+def test_whitespace_before_comment(parallel, read_tab):
     """
     Readers that don't strip whitespace from data (Tab, RDB)
     should still treat lines with leading whitespace and then
@@ -691,8 +788,9 @@ def test_whitespace_before_comment(parallel):
     expected = Table([[1], [2], [3]], names=('a', 'b', 'c'))
     assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_strip_line_trailing_whitespace(parallel):
+def test_strip_line_trailing_whitespace(parallel, read_basic):
     """
     Readers that strip whitespace from lines should ignore
     trailing whitespace after the last data value of each
@@ -708,8 +806,9 @@ def test_strip_line_trailing_whitespace(parallel):
     expected = Table([[1, 4], [2, 5], [3, 6]], names=('a', 'b', 'c'))
     assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_no_data(parallel):
+def test_no_data(parallel, read_basic):
     """
     As long as column names are supplied, the C reader
     should return an empty table in the absence of data.
@@ -721,8 +820,9 @@ def test_no_data(parallel):
     table = read_basic('a b c\n1 2 3', data_start=2, parallel=parallel)
     assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_line_endings(parallel):
+def test_line_endings(parallel, read_basic, read_commented_header, read_rdb):
     """
     Make sure the fast reader accepts CR and CR+LF
     as newlines.
@@ -745,8 +845,9 @@ def test_line_endings(parallel):
         table = read_rdb(text.replace('\n', newline), parallel=parallel)
         assert_table_equal(table, expected)
 
+
 @pytest.mark.parametrize("parallel", [True, False])
-def test_store_comments(parallel):
+def test_store_comments(parallel, read_basic):
     """
     Make sure that the output Table produced by the fast
     reader stores any comment lines in its meta attribute.
@@ -764,7 +865,7 @@ a b c
                  ['header comment', 'comment 2', 'comment 3'])
 
 @pytest.mark.parametrize("parallel", [True, False])
-def test_empty_quotes(parallel):
+def test_empty_quotes(parallel, read_basic):
     """
     Make sure the C reader doesn't segfault when the
     input data contains empty quotes. [#3407]
@@ -772,3 +873,16 @@ def test_empty_quotes(parallel):
     table = read_basic('a b\n1 ""\n2 ""', parallel=parallel)
     expected = Table([[1, 2], [0, 0]], names=('a', 'b'))
     assert_table_equal(table, expected)
+
+@pytest.mark.parametrize("parallel", [True, False])
+def test_fast_tab_with_names(parallel, read_tab):
+    """
+    Make sure the C reader doesn't segfault when the header for the
+    first column is missing [#3545]
+    """
+    content = """#
+\tdecDeg\tRate_pn_offAxis\tRate_mos2_offAxis\tObsID\tSourceID\tRADeg\tversion\tCounts_pn\tRate_pn\trun\tRate_mos1\tRate_mos2\tInserted_pn\tInserted_mos2\tbeta\tRate_mos1_offAxis\trcArcsec\tname\tInserted\tCounts_mos1\tInserted_mos1\tCounts_mos2\ty\tx\tCounts\toffAxis\tRot
+-3.007559\t0.0000\t0.0010\t0013140201\t0\t213.462574\t0\t2\t0.0002\t0\t0.0001\t0.0001\t0\t1\t0.66\t0.0217\t3.0\tfakeXMMXCS J1413.8-0300\t3\t1\t2\t1\t398.000\t127.000\t5\t13.9\t72.3\t"""
+    head = ['A{0}'.format(i) for i in range(28)]
+    table = read_tab(content, data_start=1,
+                     parallel=parallel, names=head)
