@@ -14,6 +14,7 @@ import glob
 import inspect
 import io
 import textwrap
+import os.path
 
 from collections import defaultdict
 from itertools import islice
@@ -28,7 +29,7 @@ from ...utils import indent
 from .card import Card, BLANK_CARD
 from .header import Header
 # HDUList is used in one of the doctests
-from .hdu.hdulist import fitsopen, HDUList  # pylint: disable=W0611
+from .hdu.hdulist import fitsopen  # pylint: disable=W0611
 from .hdu.table import _TableLikeHDU
 
 
@@ -92,16 +93,17 @@ class _BaseDiff(object):
     @classmethod
     def fromdiff(cls, other, a, b):
         """
-        Returns a new Diff object of a specfic subclass from an existing diff
+        Returns a new Diff object of a specific subclass from an existing diff
         object, passing on the values for any arguments they share in common
         (such as ignore_keywords).
 
         For example::
 
-            >>> hdul1, hdul2 = HDUList(), HDUList()
-            >>> headera, headerb = Header(), Header()
-            >>> fd = FITSDiff(hdul1, hdul2, ignore_keywords=['*'])
-            >>> hd = HeaderDiff.fromdiff(fd, headera, headerb)
+            >>> from astropy.io import fits
+            >>> hdul1, hdul2 = fits.HDUList(), fits.HDUList()
+            >>> headera, headerb = fits.Header(), fits.Header()
+            >>> fd = fits.FITSDiff(hdul1, hdul2, ignore_keywords=['*'])
+            >>> hd = fits.HeaderDiff.fromdiff(fd, headera, headerb)
             >>> list(hd.ignore_keywords)
             ['*']
         """
@@ -129,7 +131,7 @@ class _BaseDiff(object):
         return not any(getattr(self, attr) for attr in self.__dict__
                        if attr.startswith('diff_'))
 
-    def report(self, fileobj=None, indent=0):
+    def report(self, fileobj=None, indent=0, clobber=False):
         """
         Generates a text report on the differences (if any) between two
         objects, and either returns it as a string or writes it to a file-like
@@ -137,28 +139,46 @@ class _BaseDiff(object):
 
         Parameters
         ----------
-        fileobj : file-like object or None, optional
+        fileobj : file-like object, string, or None (optional)
             If `None`, this method returns the report as a string. Otherwise it
             returns `None` and writes the report to the given file-like object
-            (which must have a ``.write()`` method at a minimum).
+            (which must have a ``.write()`` method at a minimum), or to a new
+            file at the path specified.
 
         indent : int
             The number of 4 space tabs to indent the report.
 
+        clobber : bool
+            Whether the report output should overwrite an existing file, when
+            fileobj is specified as a path.
+
         Returns
         -------
         report : str or None
         """
 
         return_string = False
-        if fileobj is None:
+        filepath = None
+
+        if isinstance(fileobj, string_types):
+            if os.path.exists(fileobj) and not clobber:
+                raise IOError("File {0} exists, aborting (pass in "
+                              "clobber=True to overwrite)".format(fileobj))
+            else:
+                filepath = fileobj
+                fileobj = open(filepath, 'w')
+        elif fileobj is None:
             fileobj = io.StringIO()
             return_string = True
 
         self._fileobj = fileobj
         self._indent = indent  # This is used internally by _writeln
 
-        self._report()
+        try:
+            self._report()
+        finally:
+            if filepath:
+                fileobj.close()
 
         if return_string:
             return fileobj.getvalue()
@@ -218,7 +238,7 @@ class FITSDiff(_BaseDiff):
             differences.  Though the count of differences is the same either
             way, this allows controlling the number of different values that
             are kept in memory or output.  If a negative value is given, then
-            numdifs is treated as unlimited (default: 10).
+            numdiffs is treated as unlimited (default: 10).
 
         tolerance : float, optional
             The relative difference to allow when comparing two float values
@@ -928,7 +948,7 @@ class TableDataDiff(_BaseDiff):
       objects.
 
     - ``diff_column_attributes``: Lists columns that are in both tables but
-      have different secondard attributes, such as TUNIT or TDISP.  The format
+      have different secondary attributes, such as TUNIT or TDISP.  The format
       is a list of 2-tuples: The first a tuple of the column name and the
       attribute, the second a tuple of the different values.
 
