@@ -11,15 +11,14 @@ import warnings
 import numpy as np
 
 from .. import conf
-from ..card import Card
 from ..file import _File
-from ..header import Header, HEADER_END_RE, _pad_length
+from ..header import Header, _pad_length
 from ..util import (_is_int, _is_pseudo_unsigned, _unsigned_zero,
-                    itersubclasses, encode_ascii, decode_ascii,
+                    itersubclasses, decode_ascii,
                     _get_array_mmap, _array_to_file, first)
 from ..verify import _Verify, _ErrList
 
-from ....extern.six import string_types, next
+from ....extern.six import string_types
 from ....utils import lazyproperty, deprecated
 from ....utils.compat import ignored
 from ....utils.exceptions import AstropyUserWarning
@@ -320,7 +319,7 @@ class _BaseHDU(object):
         Parameters
         ----------
         data : str, bytearray, memoryview, ndarray
-           A byte string contining the HDU's header and data.
+           A byte string containing the HDU's header and data.
 
         checksum : bool, optional
            Check the HDU's checksum and/or datasum.
@@ -757,9 +756,18 @@ class _BaseHDU(object):
         self._data_size = datsize
         self._data_replaced = False
 
-_AllHDU = _BaseHDU  # For backwards-compatibility, though nobody should have
-                    # been using this directly
+    def _close(self, closed=True):
+        # If the data was mmap'd, close the underlying mmap (this will
+        # prevent any future access to the .data attribute if there are
+        # not other references to it; if there are other references then
+        # it is up to the user to clean those up
+        if (closed and self._data_loaded and
+                _get_array_mmap(self.data) is not None):
+            del self.data
 
+# For backwards-compatibility, though nobody should have
+# been using this directly:
+_AllHDU = _BaseHDU
 
 # For convenience...
 # TODO: register_hdu could be made into a class decorator which would be pretty
@@ -1232,7 +1240,7 @@ class _ValidHDU(_BaseHDU, _Verify):
             err_text = "'%s' card does not exist." % keyword
             fix_text = "Fixed by inserting a new '%s' card." % keyword
             if fixable:
-                # use repr to accomodate both string and non-string types
+                # use repr to accommodate both string and non-string types
                 # Boolean is also OK in this constructor
                 card = (keyword, fix_value)
 
@@ -1383,7 +1391,7 @@ class _ValidHDU(_BaseHDU, _Verify):
         Verify that the value in the ``DATASUM`` keyword matches the value
         calculated for the ``DATASUM`` of the current HDU data.
 
-        blocking: str, optional
+        blocking : str, optional
             "standard" or "nonstandard", compute sum 2880 bytes at a time, or
             not
 
@@ -1413,7 +1421,7 @@ class _ValidHDU(_BaseHDU, _Verify):
         Verify that the value in the ``CHECKSUM`` keyword matches the
         value calculated for the current HDU CHECKSUM.
 
-        blocking: str, optional
+        blocking : str, optional
             "standard" or "nonstandard", compute sum 2880 bytes at a time, or
             not
 
@@ -1724,7 +1732,7 @@ class ExtensionHDU(_ValidHDU):
                        1, option, errs)
 
         return errs
-# For backwards compatilibity, though this needs to be deprecated
+# For backwards compatibility, though this needs to be deprecated
 # TODO: Mark this as deprecated
 _ExtensionHDU = ExtensionHDU
 
