@@ -4,10 +4,8 @@ from __future__ import with_statement
 
 import os
 import signal
-import sys
-import warnings
 
-from ....tests.helper import pytest
+from ....tests.helper import pytest, catch_warnings
 from ..util import ignore_sigint
 
 from . import FitsTestCase
@@ -18,13 +16,13 @@ class TestUtils(FitsTestCase):
     def test_ignore_sigint(self):
         @ignore_sigint
         def test():
-            with warnings.catch_warnings(record=True) as w:
+            with catch_warnings(UserWarning) as w:
                 pid = os.getpid()
                 os.kill(pid, signal.SIGINT)
                 # One more time, for good measure
                 os.kill(pid, signal.SIGINT)
-                assert len(w) == 2
-                assert (str(w[0].message) ==
-                        'KeyboardInterrupt ignored until test is complete!')
+            assert len(w) == 2
+            assert (str(w[0].message) ==
+                    'KeyboardInterrupt ignored until test is complete!')
 
         pytest.raises(KeyboardInterrupt, test)
