@@ -37,7 +37,7 @@ from ..utils.codegen import make_function_with_signature
 from ..utils.exceptions import AstropyDeprecationWarning
 from .utils import (array_repr_oneline, check_broadcast, combine_labels,
                     make_binary_operator_eval, ExpressionTree,
-                    IncompatibleShapeError)
+                    IncompatibleShapeError, AliasDict)
 
 from .parameters import Parameter, InputParameterError
 
@@ -494,7 +494,7 @@ class Model(object):
     True
     """
 
-    parameter_constraints = ('fixed', 'tied', 'bounds')
+    parameter_constraints = Parameter.constraints
     """
     Primarily for informational purposes, these are the types of constraints
     that can be set on a model's parameters.
@@ -864,7 +864,6 @@ class Model(object):
         """
 
         # Basically this is an alternative __init__
-        # TODO: Support constraints properly
         if isinstance(self, type):
             # self is a class, not an instance
             needs_initialization = True
@@ -874,13 +873,21 @@ class Model(object):
             needs_initialization = False
             self = self.copy()
 
-        self._initialize_constraints({})
+        aliases = dict(zip(self.param_names, param_names))
+        # This is basically an alternative _initialize_constraints
+        constraints = {}
+        for cons_type in self.parameter_constraints:
+            orig = existing._constraints[cons_type]
+            constraints[cons_type] = AliasDict(orig, aliases)
+
+        self._constraints = constraints
+
         self._n_models = existing._n_models
         self._model_set_axis = existing._model_set_axis
         self._parameters = existing._parameters
 
         self._param_metrics = defaultdict(dict)
-        for param_a, param_b in zip(self.param_names, param_names):
+        for param_a, param_b in six.iteritems(aliases):
             # Take the param metrics info for the giving parameters in the
             # existing model, and hand them to the appropriate parameters in
             # the new model
@@ -906,7 +913,7 @@ class Model(object):
         # Pop any constraints off the keyword arguments
         for constraint in self.parameter_constraints:
             values = kwargs.pop(constraint, {})
-            self._constraints[constraint] = values
+            self._constraints[constraint] = values.copy()
 
             # Update with default parameter constraints
             for param_name in self.param_names:
@@ -1653,7 +1660,18 @@ class _CompoundModelMeta(_ModelMeta):
             # Both models used in the operator were already instantiated models,
             # not model *classes*.  As such it's not particularly useful to return
             # the class itself, but to instead produce a new instance:
-            return new_cls()
+            instance = new_cls()
+
+            # Workaround for https://github.com/astropy/astropy/issues/3542
+            # TODO: Any effort to restructure the tree-like data structure for
+            # compound models should try to obviate this workaround--if
+            # intermediate compound models are stored in the tree as well then
+            # we can immediately check for custom inverses on sub-models when
+            # computing the inverse
+            instance._custom_inverse = mcls._make_custom_inverse(
+                    operator, left, right)
+
+            return instance
 
         # Otherwise return the new uninstantiated class itself
         return new_cls
@@ -1668,6 +1686,48 @@ class _CompoundModelMeta(_ModelMeta):
         # _ModelMeta._handle_backwards_compat
         return
 
+    @classmethod
+    def _make_custom_inverse(mcls, operator, left, right):
+        """
+        Generates an inverse `Model` for this `_CompoundModel` when either
+        model in the operation has a *custom inverse* that was manually
+        assigned by the user.
+
+        If either model has a custom inverse, and in particular if another
+        `_CompoundModel` has a custom inverse, then none of that model's
+        sub-models should be considered at all when computing the inverse.
+        So in that case we just compute the inverse ahead of time and set
+        it as the new compound model's custom inverse.
+
+        Note, this use case only applies when combining model instances,
+        since model classes don't currently have a notion of a "custom
+        inverse" (though it could probably be supported by overriding the
+        class's inverse property).
+
+        TODO: Consider fixing things so the aforementioned class-based case
+        works as well.  However, for the present purposes this is good enough.
+        """
+
+        if not (operator in ('&', '|') and
+                (left._custom_inverse or right._custom_inverse)):
+            # These are the only operators that support an inverse right now
+            return None
+
+        try:
+            left_inv = left.inverse
+            right_inv = right.inverse
+        except NotImplementedError:
+            # If either inverse is undefined then just return False; this
+            # means the normal _CompoundModel.inverse routine will fail
+            # naturally anyways, since it requires all sub-models to have
+            # an inverse defined
+            return None
+
+        if operator == '&':
+            return left_inv & right_inv
+        else:
+            return right_inv | left_inv
+
     # TODO: Perhaps, just perhaps, the post-order (or ???-order) ordering of
     # leaf nodes is something the ExpressionTree class itself could just know
     def _get_submodels(cls):
@@ -1728,10 +1788,15 @@ class _CompoundModelMeta(_ModelMeta):
                 else:
                     default = orig_param.default
 
+                # Copy constraints
+                constraints = dict((key, getattr(orig_param, key))
+                                   for key in Model.parameter_constraints)
+
                 # Note: Parameter.copy() returns a new unbound Parameter, never
                 # a bound Parameter even if submodel is a Model instance (as
                 # opposed to a Model subclass)
-                new_param = orig_param.copy(name=param_name, default=default)
+                new_param = orig_param.copy(name=param_name, default=default,
+                                            **constraints)
 
             setattr(cls, param_name, new_param)
 
@@ -1927,7 +1992,12 @@ class _CompoundModel(Model):
     _submodels = None
 
     def __getattr__(self, attr):
-        return getattr(self.__class__, attr)
+        value = getattr(self.__class__, attr)
+        if hasattr(value, '__get__'):
+            # Object is a descriptor, so we should really return the result of
+            # its __get__
+            value = value.__get__(self, self.__class__)
+        return value
 
     def __getitem__(self, index):
         index = self.__class__._normalize_index(index)
@@ -2201,6 +2271,11 @@ def _prepare_inputs_single_model(model, params, inputs, **kwargs):
         else:
             # Extend the broadcasts list to include shapes for all outputs
             extra_outputs = model.n_outputs - model.n_inputs
+            if not broadcasts:
+                # If there were no inputs then the broadcasts list is empty
+                # just add a None since there is no broadcasting of outputs and
+                # inputs necessary (see _prepare_outputs_single_model)
+                broadcasts.append(None)
             broadcasts.extend([broadcasts[0]] * extra_outputs)
 
     return inputs, (broadcasts,)
@@ -2285,6 +2360,9 @@ def _prepare_inputs_model_set(model, params, inputs, n_models, model_set_axis,
         pivots.append(pivot)
         reshaped.append(new_input)
 
+    if model.n_inputs < model.n_outputs:
+        pivots.extend([model_set_axis] * (model.n_outputs - model.n_inputs))
+
     return reshaped, (pivots,)
 
 
@@ -2348,7 +2426,7 @@ def _validate_input_shapes(inputs, argnames, n_models, model_set_axis,
 
     try:
         input_broadcast = check_broadcast(*all_shapes)
-    except IncompatibleShapesError as exc:
+    except IncompatibleShapeError as exc:
         shape_a, shape_a_idx, shape_b, shape_b_idx = exc.args
         arg_a = argnames[shape_a_idx]
         arg_b = argnames[shape_b_idx]
