@@ -5,21 +5,19 @@
 from __future__ import (absolute_import, unicode_literals, division,
                         print_function)
 
-import collections
-
-from textwrap import dedent
+import inspect
 
 import numpy as np
 
-from .core import (Fittable1DModel, Fittable2DModel,
-                   Model, format_input, ModelDefinitionError)
+from .core import (Fittable1DModel, Fittable2DModel, Model,
+                   ModelDefinitionError, custom_model)
 from .parameters import Parameter, InputParameterError
-from ..utils import find_current_module
+from ..utils import deprecated
 from ..extern import six
 
 __all__ = sorted([
-    'AiryDisk2D', 'Beta1D', 'Beta2D', 'Box1D',
-    'Box2D', 'Const1D', 'Const2D', 'Disk2D',
+    'AiryDisk2D', 'Moffat1D', 'Moffat2D', 'Box1D',
+    'Box2D', 'Const1D', 'Const2D', 'Ellipse2D', 'Disk2D',
     'Gaussian1D', 'GaussianAbsorption1D', 'Gaussian2D', 'Linear1D',
     'Lorentz1D', 'MexicanHat1D', 'MexicanHat2D', 'Scale', 'Redshift', 'Shift',
     'Sine1D', 'Trapezoid1D', 'TrapezoidDisk2D', 'Ring2D',
@@ -87,19 +85,15 @@ class Gaussian1D(Fittable1DModel):
 
     See Also
     --------
-    Gaussian2D, Box1D, Beta1D, Lorentz1D
+    Gaussian2D, Box1D, Moffat1D, Lorentz1D
     """
 
-    amplitude = Parameter()
-    mean = Parameter()
-    stddev = Parameter()
-
-    def __init__(self, amplitude, mean, stddev, **kwargs):
-        super(Gaussian1D, self).__init__(
-            amplitude=amplitude, mean=mean, stddev=stddev, **kwargs)
+    amplitude = Parameter(default=1)
+    mean = Parameter(default=0)
+    stddev = Parameter(default=1)
 
     @staticmethod
-    def eval(x, amplitude, mean, stddev):
+    def evaluate(x, amplitude, mean, stddev):
         """
         Gaussian1D model function.
         """
@@ -142,20 +136,16 @@ class GaussianAbsorption1D(Fittable1DModel):
     Gaussian1D
     """
 
-    amplitude = Parameter()
-    mean = Parameter()
-    stddev = Parameter()
-
-    def __init__(self, amplitude, mean, stddev, **kwargs):
-        super(GaussianAbsorption1D, self).__init__(
-            amplitude=amplitude, mean=mean, stddev=stddev, **kwargs)
+    amplitude = Parameter(default=1)
+    mean = Parameter(default=0)
+    stddev = Parameter(default=1)
 
     @staticmethod
-    def eval(x, amplitude, mean, stddev):
+    def evaluate(x, amplitude, mean, stddev):
         """
         GaussianAbsorption1D model function.
         """
-        return 1.0 - Gaussian1D.eval(x, amplitude, mean, stddev)
+        return 1.0 - Gaussian1D.evaluate(x, amplitude, mean, stddev)
 
     @staticmethod
     def fit_deriv(x, amplitude, mean, stddev):
@@ -180,16 +170,16 @@ class Gaussian2D(Fittable2DModel):
     y_mean : float
         Mean of the Gaussian in y.
     x_stddev : float
-        Standard deviation of the Gaussian in x.
+        Standard deviation of the Gaussian in x before rotating by theta.
         ``x_stddev`` and ``y_stddev`` must be specified unless a covariance
         matrix (``cov_matrix``) is input.
     y_stddev : float
-        Standard deviation of the Gaussian in y.
+        Standard deviation of the Gaussian in y before rotating by theta.
         ``x_stddev`` and ``y_stddev`` must be specified unless a covariance
         matrix (``cov_matrix``) is input.
     theta : float, optional
         Rotation angle in radians. The rotation angle increases
-        counterclockwise.
+        counterclockwise, from the positive x-axis.
     cov_matrix : ndarray, optional
         A 2x2 covariance matrix. If specified, overrides the ``x_stddev``,
         ``y_stddev``, and ``theta`` specification.
@@ -237,21 +227,23 @@ class Gaussian2D(Fittable2DModel):
 
     See Also
     --------
-    Gaussian1D, Box2D, Beta2D
+    Gaussian1D, Box2D, Moffat2D
 
     References
     ----------
     .. [1] http://en.wikipedia.org/wiki/Gaussian_function
     """
 
-    amplitude = Parameter()
-    x_mean = Parameter()
-    y_mean = Parameter()
-    x_stddev = Parameter()
-    y_stddev = Parameter()
-    theta = Parameter()
+    amplitude = Parameter(default=1)
+    x_mean = Parameter(default=0)
+    y_mean = Parameter(default=0)
+    x_stddev = Parameter(default=1)
+    y_stddev = Parameter(default=1)
+    theta = Parameter(default=0)
 
-    def __init__(self, amplitude, x_mean, y_mean, x_stddev=None, y_stddev=None,
+
+    def __init__(self, amplitude=amplitude.default, x_mean=x_mean.default,
+                 y_mean=y_mean.default, x_stddev=None, y_stddev=None,
                  theta=0.0, cov_matrix=None, **kwargs):
         if y_stddev is None and cov_matrix is None:
             raise InputParameterError(
@@ -268,9 +260,19 @@ class Gaussian2D(Fittable2DModel):
 
         # Compute principle coordinate system transformation
         elif cov_matrix is not None:
+            if (x_stddev is not None or y_stddev is not None):
+                raise InputParameterError(
+                    "Cannot specify both cov_matrix and x/y_stddev")
+            # Compute principle coordinate system transformation
             cov_matrix = np.array(cov_matrix)
+
             if cov_matrix.shape != (2, 2):
+                # TODO: Maybe it should be possible for the covariance matrix
+                # to be some (x, y, ..., z, 2, 2) array to be broadcast with
+                # other parameters of shape (x, y, ..., z)
+                # But that's maybe a special case to work out if/when needed
                 raise ValueError("Covariance matrix must be 2x2")
+
             eig_vals, eig_vecs = np.linalg.eig(cov_matrix)
             x_stddev, y_stddev = np.sqrt(eig_vals)
             y_vec = eig_vecs[:, 0]
@@ -281,7 +283,7 @@ class Gaussian2D(Fittable2DModel):
             x_stddev=x_stddev, y_stddev=y_stddev, theta=theta, **kwargs)
 
     @staticmethod
-    def eval(x, y, amplitude, x_mean, y_mean, x_stddev, y_stddev, theta):
+    def evaluate(x, y, amplitude, x_mean, y_mean, x_stddev, y_stddev, theta):
         """Two dimensional Gaussian function"""
 
         cost2 = np.cos(theta) ** 2
@@ -351,37 +353,24 @@ class Shift(Model):
 
     Parameters
     ----------
-    offsets : float or a list of floats
-        offsets to be applied to a coordinate
-        if a list - each value in the list is an offset to be applied to a
-        column in the input coordinate array
+    offset : float
+        Offset to add to a coordinate.
     """
 
-    offsets = Parameter()
+    inputs = ('x',)
+    outputs = ('x',)
 
-    def __init__(self, offsets, **kwargs):
-        super(Shift, self).__init__(offsets, **kwargs)
+    offset = Parameter(default=0)
 
-    # TODO: Might need to do some work to ensure that cases like this work
-    # consistently.  Should iterating over self.offsets mean iterating over its
-    # parameter sets?  Maybe something like this should just work seamlessly
+    @property
     def inverse(self):
         inv = self.copy()
-        inv.offsets *= -1
+        inv.offset *= -1
         return inv
 
-    @format_input
-    def __call__(self, x, model_set_axis=None):
-        """
-        Transforms data using this model.
-
-        Parameters
-        ----------
-        x : array like or a number
-            input
-        """
-
-        return self.offsets + x
+    @staticmethod
+    def evaluate(x, offset):
+        return x + offset
 
 
 class Scale(Model):
@@ -390,33 +379,25 @@ class Scale(Model):
 
     Parameters
     ----------
-    factors : float or a list of floats
-        scale for a coordinate
+    factor : float
+        Factor by which to scale a coordinate.
     """
 
-    factors = Parameter()
+    inputs = ('x',)
+    outputs = ('x',)
+
+    factor = Parameter(default=1)
     linear = True
 
-    def __init__(self, factors, **kwargs):
-        super(Scale, self).__init__(factors, **kwargs)
-
+    @property
     def inverse(self):
         inv = self.copy()
-        inv.factors = 1 / self.factors
+        inv.factor = 1 / self.factor
         return inv
 
-    @format_input
-    def __call__(self, x, model_set_axis=None):
-        """
-        Transforms data using this model.
-
-        Parameters
-        ----------
-        x : array like or a number
-            input
-        """
-
-        return self.factors * x
+    @staticmethod
+    def evaluate(x, factor):
+        return factor * x
 
 
 class Redshift(Fittable1DModel):
@@ -435,14 +416,13 @@ class Redshift(Fittable1DModel):
         .. math:: \\lambda_{obs} = (1 + z) \\lambda_{rest}
 
     """
-    z = Parameter(description='redshift')
 
-    def __init__(self, z, **kwargs):
-        super(Redshift, self).__init__(z=z, **kwargs)
+    z = Parameter(description='redshift', default=0)
 
     @staticmethod
-    def eval(x, z):
+    def evaluate(x, z):
         """One dimensional Redshift model function"""
+
         return (1 + z) * x
 
     @staticmethod
@@ -451,6 +431,7 @@ class Redshift(Fittable1DModel):
         d_z = x
         return [d_z]
 
+    @property
     def inverse(self):
         """Inverse Redshift model"""
 
@@ -482,15 +463,11 @@ class Sine1D(Fittable1DModel):
         .. math:: f(x) = A \\sin(2 \\pi f x)
     """
 
-    amplitude = Parameter()
-    frequency = Parameter()
-
-    def __init__(self, amplitude, frequency, **kwargs):
-        super(Sine1D, self).__init__(
-            amplitude=amplitude, frequency=frequency, **kwargs)
+    amplitude = Parameter(default=1)
+    frequency = Parameter(default=1)
 
     @staticmethod
-    def eval(x, amplitude, frequency):
+    def evaluate(x, amplitude, frequency):
         """One dimensional Sine model function"""
 
         return amplitude * np.sin(2 * np.pi * frequency * x)
@@ -528,16 +505,12 @@ class Linear1D(Fittable1DModel):
         .. math:: f(x) = a x + b
     """
 
-    slope = Parameter()
-    intercept = Parameter()
+    slope = Parameter(default=1)
+    intercept = Parameter(default=0)
     linear = True
 
-    def __init__(self, slope, intercept, **kwargs):
-        super(Linear1D, self).__init__(
-            slope=slope, intercept=intercept, **kwargs)
-
     @staticmethod
-    def eval(x, slope, intercept):
+    def evaluate(x, slope, intercept):
         """One dimensional Line model function"""
 
         return slope * x + intercept
@@ -577,16 +550,12 @@ class Lorentz1D(Fittable1DModel):
         f(x) = \\frac{A \\gamma^{2}}{\\gamma^{2} + \\left(x - x_{0}\\right)^{2}}
     """
 
-    amplitude = Parameter()
-    x_0 = Parameter()
-    fwhm = Parameter()
-
-    def __init__(self, amplitude, x_0, fwhm, **kwargs):
-        super(Lorentz1D, self).__init__(
-            amplitude=amplitude, x_0=x_0, fwhm=fwhm, **kwargs)
+    amplitude = Parameter(default=1)
+    x_0 = Parameter(default=0)
+    fwhm = Parameter(default=1)
 
     @staticmethod
-    def eval(x, amplitude, x_0, fwhm):
+    def evaluate(x, amplitude, x_0, fwhm):
         """One dimensional Lorentzian model function"""
 
         return (amplitude * ((fwhm / 2.) ** 2) / ((x - x_0) ** 2 +
@@ -594,7 +563,7 @@ class Lorentz1D(Fittable1DModel):
 
     @staticmethod
     def fit_deriv(x, amplitude, x_0, fwhm):
-        """One dimensional Lorentzian model derivative wiht respect to parameters"""
+        """One dimensional Lorentzian model derivative with respect to parameters"""
 
         d_amplitude = fwhm ** 2 / (fwhm ** 2 + (x - x_0) ** 2)
         d_x_0 = (amplitude * d_amplitude * (2 * x - 2 * x_0) /
@@ -623,17 +592,23 @@ class Const1D(Fittable1DModel):
         .. math:: f(x) = A
     """
 
-    amplitude = Parameter()
+    amplitude = Parameter(default=1)
     linear = True
 
-    def __init__(self, amplitude, **kwargs):
-        super(Const1D, self).__init__(amplitude=amplitude, **kwargs)
-
     @staticmethod
-    def eval(x, amplitude):
+    def evaluate(x, amplitude):
         """One dimensional Constant model function"""
 
-        return amplitude * np.ones_like(x)
+        if amplitude.size == 1:
+            # This is slightly faster than using ones_like and multiplying
+            x = np.empty_like(x)
+            x.fill(amplitude.item())
+        else:
+            # This case is less likely but could occur if the amplitude
+            # parameter is given an array-like value
+            x = amplitude * np.ones_like(x)
+
+        return x
 
     @staticmethod
     def fit_deriv(x, amplitude):
@@ -663,17 +638,114 @@ class Const2D(Fittable2DModel):
         .. math:: f(x, y) = A
     """
 
-    amplitude = Parameter()
+    amplitude = Parameter(default=1)
     linear = True
 
-    def __init__(self, amplitude, **kwargs):
-        super(Const2D, self).__init__(amplitude=amplitude, **kwargs)
-
     @staticmethod
-    def eval(x, y, amplitude):
+    def evaluate(x, y, amplitude):
         """Two dimensional Constant model function"""
 
-        return amplitude * np.ones_like(x)
+        if amplitude.size == 1:
+            # This is slightly faster than using ones_like and multiplying
+            x = np.empty_like(x)
+            x.fill(amplitude.item())
+        else:
+            # This case is less likely but could occur if the amplitude
+            # parameter is given an array-like value
+            x = amplitude * np.ones_like(x)
+
+        return x
+
+
+class Ellipse2D(Fittable2DModel):
+    """
+    A 2D Ellipse model.
+
+    Parameters
+    ----------
+    amplitude : float
+        Value of the ellipse.
+
+    x_0 : float
+        x position of the center of the disk.
+
+    y_0 : float
+        y position of the center of the disk.
+
+    a : float
+        The length of the semimajor axis.
+
+    b : float
+        The length of the semiminor axis.
+
+    theta : float, optional
+        The rotation angle in radians of the semimajor axis.  The
+        rotation angle increases counterclockwise from the positive x
+        axis.
+
+    See Also
+    --------
+    Disk2D, Box2D
+
+    Notes
+    -----
+    Model formula:
+
+    .. math::
+
+        f(x, y) = \\left \\{
+                    \\begin{array}{ll}
+                      \\mathrm{amplitude} & : \\left[\\frac{(x - x_0) \\cos
+                        \\theta + (y - y_0) \\sin \\theta}{a}\\right]^2 +
+                        \\left[\\frac{-(x - x_0) \\sin \\theta + (y - y_0)
+                        \\cos \\theta}{b}\\right]^2  \\leq 1 \\\\
+                      0 & : \\mathrm{otherwise}
+                    \\end{array}
+                  \\right.
+
+    Examples
+    --------
+    .. plot::
+        :include-source:
+
+        import numpy as np
+        from astropy.modeling.models import Ellipse2D
+        from astropy.coordinates import Angle
+        import matplotlib.pyplot as plt
+        import matplotlib.patches as mpatches
+        x0, y0 = 25, 25
+        a, b = 20, 10
+        theta = Angle(30, 'deg')
+        e = Ellipse2D(amplitude=100., x_0=x0, y_0=y0, a=a, b=b,
+                      theta=theta.radian)
+        y, x = np.mgrid[0:50, 0:50]
+        fig, ax = plt.subplots(1, 1)
+        ax.imshow(e(x, y), origin='lower', interpolation='none', cmap='Greys_r')
+        e2 = mpatches.Ellipse((x0, y0), 2*a, 2*b, theta.degree,
+                              edgecolor='red', facecolor='none')
+        ax.add_patch(e2)
+        plt.show()
+    """
+
+    amplitude = Parameter()
+    x_0 = Parameter()
+    y_0 = Parameter()
+    a = Parameter()
+    b = Parameter()
+    theta = Parameter()
+
+    @staticmethod
+    def evaluate(x, y, amplitude, x_0, y_0, a, b, theta):
+        """Two dimensional Ellipse model function."""
+
+        xx = x - x_0
+        yy = y - y_0
+        cost = np.cos(theta)
+        sint = np.sin(theta)
+        numerator1 = (xx * cost) + (yy * sint)
+        numerator2 = -(xx * sint) + (yy * cost)
+        in_ellipse = (((numerator1 / a) ** 2 + (numerator2 / b) ** 2) <= 1.)
+        return np.select([in_ellipse], [amplitude])
 
 
 class Disk2D(Fittable2DModel):
@@ -709,17 +781,13 @@ class Disk2D(Fittable2DModel):
                    \\right.
     """
 
-    amplitude = Parameter()
-    x_0 = Parameter()
-    y_0 = Parameter()
-    R_0 = Parameter()
-
-    def __init__(self, amplitude, x_0, y_0, R_0, **kwargs):
-        super(Disk2D, self).__init__(
-            amplitude=amplitude, x_0=x_0, y_0=y_0, R_0=R_0, **kwargs)
+    amplitude = Parameter(default=1)
+    x_0 = Parameter(default=0)
+    y_0 = Parameter(default=0)
+    R_0 = Parameter(default=1)
 
     @staticmethod
-    def eval(x, y, amplitude, x_0, y_0, R_0):
+    def evaluate(x, y, amplitude, x_0, y_0, R_0):
         """Two dimensional Disk model function"""
 
         rr = (x - x_0) ** 2 + (y - y_0) ** 2
@@ -766,25 +834,29 @@ class Ring2D(Fittable2DModel):
     Where :math:`r_{out} = r_{in} + r_{width}`.
     """
 
-    amplitude = Parameter()
-    x_0 = Parameter()
-    y_0 = Parameter()
-    r_in = Parameter()
-    width = Parameter()
+    amplitude = Parameter(default=1)
+    x_0 = Parameter(default=0)
+    y_0 = Parameter(default=0)
+    r_in = Parameter(default=1)
+    width = Parameter(default=1)
 
-    def __init__(self, amplitude, x_0, y_0, r_in, width=None, r_out=None,
-                 **kwargs):
+    def __init__(self, amplitude=amplitude.default, x_0=x_0.default,
+                 y_0=y_0.default, r_in=r_in.default, width=width.default,
+                 r_out=None, **kwargs):
         if r_out is not None:
+            if width is not None:
+                raise InputParameterError(
+                    "Cannot specify both width and outer radius separately.")
             width = r_out - r_in
-        if r_out is None and width is None:
-            raise ModelDefinitionError("Either specify width or r_out.")
+        elif width is None:
+            width = self.width.default
 
         super(Ring2D, self).__init__(
             amplitude=amplitude, x_0=x_0, y_0=y_0, r_in=r_in, width=width,
             **kwargs)
 
     @staticmethod
-    def eval(x, y, amplitude, x_0, y_0, r_in, width):
+    def evaluate(x, y, amplitude, x_0, y_0, r_in, width):
         """Two dimensional Ring model function."""
 
         rr = (x - x_0) ** 2 + (y - y_0) ** 2
@@ -837,16 +909,12 @@ class Box1D(Fittable1DModel):
                    \\right.
     """
 
-    amplitude = Parameter()
-    x_0 = Parameter()
-    width = Parameter()
-
-    def __init__(self, amplitude, x_0, width, **kwargs):
-        super(Box1D, self).__init__(
-            amplitude=amplitude, x_0=x_0, width=width, **kwargs)
+    amplitude = Parameter(default=1)
+    x_0 = Parameter(default=0)
+    width = Parameter(default=1)
 
     @staticmethod
-    def eval(x, amplitude, x_0, width):
+    def evaluate(x, amplitude, x_0, width):
         """One dimensional Box model function"""
 
         return np.select([np.logical_and(x >= x_0 - width / 2.,
@@ -857,7 +925,7 @@ class Box1D(Fittable1DModel):
     def fit_deriv(cls, x, amplitude, x_0, width):
         """One dimensional Box model derivative with respect to parameters"""
 
-        d_amplitude = cls.eval(x, 1, x_0, width)
+        d_amplitude = cls.evaluate(x, 1, x_0, width)
         d_x_0 = np.zeros_like(x)
         d_width = np.zeros_like(x)
         return [d_amplitude, d_x_0, d_width]
@@ -882,7 +950,7 @@ class Box2D(Fittable2DModel):
 
     See Also
     --------
-    Box1D, Gaussian2D, Beta2D
+    Box1D, Gaussian2D, Moffat2D
 
     Notes
     -----
@@ -900,20 +968,16 @@ class Box2D(Fittable2DModel):
 
     """
 
-    amplitude = Parameter()
-    x_0 = Parameter()
-    y_0 = Parameter()
-    x_width = Parameter()
-    y_width = Parameter()
-
-    def __init__(self, amplitude, x_0, y_0, x_width, y_width, **kwargs):
-        super(Box2D, self).__init__(
-            amplitude=amplitude, x_0=x_0, y_0=y_0, x_width=x_width,
-            y_width=y_width, **kwargs)
+    amplitude = Parameter(default=1)
+    x_0 = Parameter(default=0)
+    y_0 = Parameter(default=0)
+    x_width = Parameter(default=1)
+    y_width = Parameter(default=1)
 
     @staticmethod
-    def eval(x, y, amplitude, x_0, y_0, x_width, y_width):
+    def evaluate(x, y, amplitude, x_0, y_0, x_width, y_width):
         """Two dimensional Box model function"""
+
         x_range = np.logical_and(x >= x_0 - x_width / 2.,
                                  x <= x_0 + x_width / 2.)
         y_range = np.logical_and(y >= y_0 - y_width / 2.,
@@ -938,21 +1002,18 @@ class Trapezoid1D(Fittable1DModel):
 
     See Also
     --------
-    Box1D, Gaussian1D, Beta1D
+    Box1D, Gaussian1D, Moffat1D
     """
 
-    amplitude = Parameter()
-    x_0 = Parameter()
-    width = Parameter()
-    slope = Parameter()
-
-    def __init__(self, amplitude, x_0, width, slope, **kwargs):
-        super(Trapezoid1D, self).__init__(
-            amplitude=amplitude, x_0=x_0, width=width, slope=slope, **kwargs)
+    amplitude = Parameter(default=1)
+    x_0 = Parameter(default=0)
+    width = Parameter(default=1)
+    slope = Parameter(default=1)
 
     @staticmethod
-    def eval(x, amplitude, x_0, width, slope):
+    def evaluate(x, amplitude, x_0, width, slope):
         """One dimensional Trapezoid model function"""
+
         # Compute the four points where the trapezoid changes slope
         # x1 <= x2 <= x3 <= x4
         x2 = x_0 - width / 2.
@@ -992,19 +1053,14 @@ class TrapezoidDisk2D(Fittable2DModel):
     Disk2D, Box2D
     """
 
-    amplitude = Parameter()
-    x_0 = Parameter()
-    y_0 = Parameter()
-    R_0 = Parameter()
-    slope = Parameter()
-
-    def __init__(self, amplitude, x_0, y_0, R_0, slope, **kwargs):
-        super(TrapezoidDisk2D, self).__init__(
-            amplitude=amplitude, x_0=x_0, y_0=y_0, R_0=R_0, slope=slope,
-            **kwargs)
+    amplitude = Parameter(default=1)
+    x_0 = Parameter(default=0)
+    y_0 = Parameter(default=0)
+    R_0 = Parameter(default=1)
+    slope = Parameter(default=1)
 
     @staticmethod
-    def eval(x, y, amplitude, x_0, y_0, R_0, slope):
+    def evaluate(x, y, amplitude, x_0, y_0, R_0, slope):
         """Two dimensional Trapezoid Disk model function"""
 
         r = np.sqrt((x - x_0) ** 2 + (y - y_0) ** 2)
@@ -1043,16 +1099,12 @@ class MexicanHat1D(Fittable1DModel):
 
     """
 
-    amplitude = Parameter()
-    x_0 = Parameter()
-    sigma = Parameter()
-
-    def __init__(self, amplitude, x_0, sigma, **kwargs):
-        super(MexicanHat1D, self).__init__(
-            amplitude=amplitude, x_0=x_0, sigma=sigma, **kwargs)
+    amplitude = Parameter(default=1)
+    x_0 = Parameter(default=0)
+    sigma = Parameter(default=1)
 
     @staticmethod
-    def eval(x, amplitude, x_0, sigma):
+    def evaluate(x, amplitude, x_0, sigma):
         """One dimensional Mexican Hat model function"""
 
         xx_ww = (x - x_0) ** 2 / (2 * sigma ** 2)
@@ -1090,17 +1142,13 @@ class MexicanHat2D(Fittable2DModel):
         - \\left(y - y_{0}\\right)^{2}}{2 \\sigma^{2}}}
     """
 
-    amplitude = Parameter()
-    x_0 = Parameter()
-    y_0 = Parameter()
-    sigma = Parameter()
-
-    def __init__(self, amplitude, x_0, y_0, sigma, **kwargs):
-        super(MexicanHat2D, self).__init__(
-            amplitude=amplitude, x_0=x_0, y_0=y_0, sigma=sigma, **kwargs)
+    amplitude = Parameter(default=1)
+    x_0 = Parameter(default=0)
+    y_0 = Parameter(default=0)
+    sigma = Parameter(default=1)
 
     @staticmethod
-    def eval(x, y, amplitude, x_0, y_0, sigma):
+    def evaluate(x, y, amplitude, x_0, y_0, sigma):
         """Two dimensional Mexican Hat model function"""
 
         rr_ww = ((x - x_0) ** 2 + (y - y_0) ** 2) / (2 * sigma ** 2)
@@ -1150,13 +1198,14 @@ class AiryDisk2D(Fittable2DModel):
     .. [1] http://en.wikipedia.org/wiki/Airy_disk
     """
 
-    amplitude = Parameter()
-    x_0 = Parameter()
-    y_0 = Parameter()
-    radius = Parameter()
+    amplitude = Parameter(default=1)
+    x_0 = Parameter(default=0)
+    y_0 = Parameter(default=0)
+    radius = Parameter(default=1)
     _j1 = None
 
-    def __init__(self, amplitude, x_0, y_0, radius, **kwargs):
+    def __init__(self, amplitude=amplitude.default, x_0=x_0.default,
+                 y_0=y_0.default, radius=radius.default, **kwargs):
         if self._j1 is None:
             try:
                 from scipy.special import j1, jn_zeros
@@ -1169,6 +1218,9 @@ class AiryDisk2D(Fittable2DModel):
         super(AiryDisk2D, self).__init__(
             amplitude=amplitude, x_0=x_0, y_0=y_0, radius=radius, **kwargs)
 
+    # TODO: Why does this particular model have its own special __deepcopy__
+    # and __copy__?  If it has anything to do with the use of the j_1 function
+    # that should be reworked.
     def __deepcopy__(self, memo):
         new_model = self.__class__(self.amplitude.value, self.x_0.value,
                                    self.y_0.value, self.radius.value)
@@ -1180,7 +1232,7 @@ class AiryDisk2D(Fittable2DModel):
         return new_model
 
     @classmethod
-    def eval(cls, x, y, amplitude, x_0, y_0, radius):
+    def evaluate(cls, x, y, amplitude, x_0, y_0, radius):
         """Two dimensional Airy model function"""
 
         r = np.sqrt((x - x_0) ** 2 + (y - y_0) ** 2) / (radius / cls._rz)
@@ -1193,20 +1245,20 @@ class AiryDisk2D(Fittable2DModel):
         return z
 
 
-class Beta1D(Fittable1DModel):
+class Moffat1D(Fittable1DModel):
     """
-    One dimensional Beta model.
+    One dimensional Moffat model.
 
     Parameters
     ----------
     amplitude : float
         Amplitude of the model.
     x_0 : float
-        x position of the maximum of the Beta model.
+        x position of the maximum of the Moffat model.
     gamma : float
-        Core width of the Beta model.
+        Core width of the Moffat model.
     alpha : float
-        Power index of the beta model.
+        Power index of the Moffat model.
 
     See Also
     --------
@@ -1221,24 +1273,20 @@ class Beta1D(Fittable1DModel):
         f(x) = A \\left(1 + \\frac{\\left(x - x_{0}\\right)^{2}}{\\gamma^{2}}\\right)^{- \\alpha}
     """
 
-    amplitude = Parameter()
-    x_0 = Parameter()
-    gamma = Parameter()
-    alpha = Parameter()
-
-    def __init__(self, amplitude, x_0, gamma, alpha, **kwargs):
-        super(Beta1D, self).__init__(
-            amplitude=amplitude, x_0=x_0, gamma=gamma, alpha=alpha, **kwargs)
+    amplitude = Parameter(default=1)
+    x_0 = Parameter(default=0)
+    gamma = Parameter(default=1)
+    alpha = Parameter(default=1)
 
     @staticmethod
-    def eval(x, amplitude, x_0, gamma, alpha):
-        """One dimensional Beta model function"""
+    def evaluate(x, amplitude, x_0, gamma, alpha):
+        """One dimensional Moffat model function"""
 
         return amplitude * (1 + ((x - x_0) / gamma) ** 2) ** (-alpha)
 
     @staticmethod
     def fit_deriv(x, amplitude, x_0, gamma, alpha):
-        """One dimensional Beta model derivative with respect to parameters"""
+        """One dimensional Moffat model derivative with respect to parameters"""
 
         d_A = (1 + (x - x_0) ** 2 / gamma ** 2) ** (-alpha)
         d_x_0 = (-amplitude * alpha * d_A * (-2 * x + 2 * x_0) /
@@ -1249,22 +1297,22 @@ class Beta1D(Fittable1DModel):
         return [d_A, d_x_0, d_gamma, d_alpha]
 
 
-class Beta2D(Fittable2DModel):
+class Moffat2D(Fittable2DModel):
     """
-    Two dimensional Beta model.
+    Two dimensional Moffat model.
 
     Parameters
     ----------
     amplitude : float
         Amplitude of the model.
     x_0 : float
-        x position of the maximum of the Beta model.
+        x position of the maximum of the Moffat model.
     y_0 : float
-        y position of the maximum of the Beta model.
+        y position of the maximum of the Moffat model.
     gamma : float
-        Core width of the Beta model.
+        Core width of the Moffat model.
     alpha : float
-        Power index of the beta model.
+        Power index of the Moffat model.
 
     See Also
     --------
@@ -1280,27 +1328,22 @@ class Beta2D(Fittable2DModel):
         \\left(y - y_{0}\\right)^{2}}{\\gamma^{2}}\\right)^{- \\alpha}
     """
 
-    amplitude = Parameter()
-    x_0 = Parameter()
-    y_0 = Parameter()
-    gamma = Parameter()
-    alpha = Parameter()
-
-    def __init__(self, amplitude, x_0, y_0, gamma, alpha, **kwargs):
-        super(Beta2D, self).__init__(
-            amplitude=amplitude, x_0=x_0, y_0=y_0, gamma=gamma, alpha=alpha,
-            **kwargs)
+    amplitude = Parameter(default=1)
+    x_0 = Parameter(default=0)
+    y_0 = Parameter(default=0)
+    gamma = Parameter(default=1)
+    alpha = Parameter(default=1)
 
     @staticmethod
-    def eval(x, y, amplitude, x_0, y_0, gamma, alpha):
-        """Two dimensional Beta model function"""
+    def evaluate(x, y, amplitude, x_0, y_0, gamma, alpha):
+        """Two dimensional Moffat model function"""
 
         rr_gg = ((x - x_0) ** 2 + (y - y_0) ** 2) / gamma ** 2
         return amplitude * (1 + rr_gg) ** (-alpha)
 
     @staticmethod
     def fit_deriv(x, y, amplitude, x_0, y_0, gamma, alpha):
-        """Two dimensional Beta model derivative with respect to parameters"""
+        """Two dimensional Moffat model derivative with respect to parameters"""
 
         rr_gg = ((x - x_0) ** 2 + (y - y_0) ** 2) / gamma ** 2
         d_A = (1 + rr_gg) ** (-alpha)
@@ -1313,118 +1356,17 @@ class Beta2D(Fittable2DModel):
         return [d_A, d_x_0, d_y_0, d_gamma, d_alpha]
 
 
+@deprecated('1.0', alternative='astropy.modeling.models.custom_model',
+            pending=True)
 def custom_model_1d(func, func_fit_deriv=None):
-    """
-    Create a one dimensional model from a user defined function. The
-    parameters of the model will be inferred from the arguments of
-    the function.
-
-    .. note::
-
-        All model parameters have to be defined as keyword arguments
-        with default values in the model function.
-
-    If you want to use parameter sets in the model, the parameters should be
-    treated as lists or arrays.
-
-    Parameters
-    ----------
-    func : function
-        Function which defines the model.  It should take one positional
-        argument (the independent variable in the model), and any number of
-        keyword arguments (the parameters).  It must return the value of the
-        model (typically as an array, but can also be a scalar for scalar
-        inputs).  This corresponds to the
-        `~astropy.modeling.Fittable1DModel.eval` method.
-    func_fit_deriv : function, optional
-        Function which defines the Jacobian derivative of the model. I.e., the
-        derivive with respect to the *parameters* of the model.  It should
-        have the same argument signature as ``func``, but should return a
-        sequence where each element of the sequence is the derivative
-        with respect to the correseponding argument. This corresponds to the
-        :meth:`~astropy.modeling.FittableModel.fit_deriv` method.
-
-
-    Examples
-    --------
-    Define a sinusoidal model function as a custom 1D model:
-
-        >>> from astropy.modeling.models import custom_model_1d
-        >>> import numpy as np
-        >>> def sine_model(x, amplitude=1., frequency=1.):
-        ...     return amplitude * np.sin(2 * np.pi * frequency * x)
-        >>> def sine_deriv(x, amplitude=1., frequency=1.):
-        ...     return 2 * np.pi * amplitude * np.cos(2 * np.pi * frequency * x)
-        >>> SineModel = custom_model_1d(sine_model, func_fit_deriv=sine_deriv)
-
-    Create an instance of the custom model and evaluate it:
-
-        >>> model = SineModel()
-        >>> model(0.25)
-        1.0
-
-    This model instance can now be used like a usual astropy model.
-    """
-
-    if not six.callable(func):
-        raise ModelDefinitionError("Not callable. Must be function")
-
-    if func_fit_deriv is not None and not six.callable(func_fit_deriv):
-        raise ModelDefinitionError(
-                "func_fit_deriv not callable. Must be function")
-
-    model_name = func.__name__
-    param_values = six.get_function_defaults(func)
-
-    # Check if all parameters are keyword arguments
+    argspec = inspect.getargspec(func)
+    param_values = argspec.defaults
     nparams = len(param_values)
 
-    if (func_fit_deriv is not None and
-            len(six.get_function_defaults(func_fit_deriv)) != nparams):
-        raise ModelDefinitionError("derivative function should accept "
-                                   "same number of parameters as func.")
-
-    func_code = six.get_function_code(func)
-    if func_code.co_argcount == nparams + 1:
-        param_names = func_code.co_varnames[1:nparams + 1]
+    if len(argspec.args) == nparams + 1:
+        param_names = argspec.args[-nparams:]
     else:
         raise ModelDefinitionError(
             "All parameters must be keyword arguments")
 
-    params = dict((name, Parameter(name, default=default))
-                  for name, default in zip(param_names, param_values))
-
-    arg_signature_1 = ', '.join('{0}=None'.format(name)
-                                for name in param_names)
-    arg_signature_2 = ', '.join('{0}={0}'.format(name)
-                                for name in param_names)
-
-    mod = find_current_module(2)
-    if mod:
-        filename = mod.__file__
-        modname = mod.__name__
-    else:
-        filename = '<string>'
-        modname = '__main__'
-
-    members = {'eval': staticmethod(func)}
-
-    eval_globals = {}
-
-    init_code_string = dedent("""
-        def __init__(self, {0}, **kwargs):
-            super(self.__class__, self).__init__({1}, **kwargs)
-    """).format(arg_signature_1, arg_signature_2)
-
-    eval(compile(init_code_string, filename, 'single'), eval_globals)
-
-    if func_fit_deriv is not None:
-        members['fit_deriv'] = staticmethod(func_fit_deriv)
-
-    members['__init__'] = eval_globals['__init__']
-    members.update(params)
-
-    cls = type(model_name, (Fittable1DModel,), members)
-    cls.__module__ = modname
-
-    return cls
+    return custom_model(func, fit_deriv=func_fit_deriv)
