@@ -12,7 +12,7 @@ import numpy as np
 from .core import FittableModel, Model
 from .functional_models import Shift
 from .parameters import Parameter
-from .utils import poly_map_domain, comb
+from .utils import poly_map_domain, comb, check_broadcast
 from ..utils import lazyproperty, indent
 
 
@@ -78,14 +78,6 @@ class PolynomialBase(FittableModel):
         else:
             super(PolynomialBase, self).__setattr__(attr, value)
 
-    def _validate_params(self, **params):
-        valid_params = set(self._param_names)
-        provided_params = set(params)
-        intersection = valid_params.intersection(provided_params)
-        if len(intersection) != len(provided_params):
-            diff = list(provided_params.difference(valid_params))
-            raise TypeError('Unrecognized input parameters: %s' % diff)
-
 
 class PolynomialModel(PolynomialBase):
     """
@@ -102,9 +94,6 @@ class PolynomialModel(PolynomialBase):
         self._order = self.get_num_coeff(self.n_inputs)
         self._param_names = self._generate_coeff_names(self.n_inputs)
 
-        if params:
-            self._validate_params(**params)
-
         super(PolynomialModel, self).__init__(
             n_models=n_models, model_set_axis=model_set_axis, name=name,
             meta=meta, **params)
@@ -126,8 +115,8 @@ class PolynomialModel(PolynomialBase):
         Return the number of coefficients in one parameter set
         """
 
-        if self.degree < 1  or self.degree > 16:
-            raise ValueError("Degree of polynomial must be 1< deg < 16")
+        if self.degree < 0:
+            raise ValueError("Degree of polynomial must be positive or null")
         # deg+1 is used to account for the difference between iraf using
         # degree and numpy using exact degree
         if ndim != 1:
@@ -209,9 +198,6 @@ class OrthoPolynomialBase(PolynomialBase):
         self.y_window = y_window
         self._param_names = self._generate_coeff_names()
 
-        if params:
-            self._validate_params(**params)
-
         super(OrthoPolynomialBase, self).__init__(
             n_models=n_models, model_set_axis=model_set_axis,
             name=name, meta=meta, **params)
@@ -313,6 +299,10 @@ class OrthoPolynomialBase(PolynomialBase):
         raise NotImplementedError("Subclasses should implement this")
 
     def evaluate(self, x, y, *coeffs):
+        if self.x_domain is not None:
+            x = poly_map_domain(x, self.x_domain, self.x_window)
+        if self.y_domain is not None:
+            y = poly_map_domain(y, self.y_domain, self.y_window)
         invcoeff = self.invlex_coeff()
         return self.imhorner(x, y, invcoeff)
 
@@ -324,10 +314,6 @@ class OrthoPolynomialBase(PolynomialBase):
 
         if x.shape != y.shape:
             raise ValueError("Expected input arrays to have the same shape")
-        if self.x_domain is not None:
-            x = poly_map_domain(x, self.x_domain, self.x_window)
-        if self.y_domain is not None:
-            y = poly_map_domain(y, self.y_domain, self.y_window)
 
         return (x, y), format_info
 
@@ -380,11 +366,12 @@ class Chebyshev1D(PolynomialModel):
 
         x = np.array(x, dtype=np.float, copy=False, ndmin=1)
         v = np.empty((self.degree + 1,) + x.shape, dtype=x.dtype)
-        v[0] = x * 0 + 1
-        x2 = 2 * x
-        v[1] = x
-        for i in range(2, self.degree + 1):
-            v[i] = v[i - 1] * x2 - v[i - 2]
+        v[0] = 1
+        if self.degree > 0:
+            x2 = 2 * x
+            v[1] = x
+            for i in range(2, self.degree + 1):
+                v[i] = v[i - 1] * x2 - v[i - 2]
         return np.rollaxis(v, 0, v.ndim)
 
     def prepare_inputs(self, x, **kwargs):
@@ -393,14 +380,12 @@ class Chebyshev1D(PolynomialModel):
 
         x = inputs[0]
 
-        if self.domain is not None:
-            x = poly_map_domain(x, self.domain, self.window)
-
         return (x,), format_info
 
-    @classmethod
-    def evaluate(cls, x, *coeffs):
-        return cls.clenshaw(x, coeffs)
+    def evaluate(self, x, *coeffs):
+        if self.domain is not None:
+            x = poly_map_domain(x, self.domain, self.window)
+        return self.clenshaw(x, coeffs)
 
     @staticmethod
     def clenshaw(x, coeffs):
@@ -458,14 +443,12 @@ class Legendre1D(PolynomialModel):
 
         x = inputs[0]
 
-        if self.domain is not None:
-            x = poly_map_domain(x, self.domain, self.window)
-
         return (x,), format_info
 
-    @classmethod
-    def evaluate(cls, x, *coeffs):
-        return cls.clenshaw(x, coeffs)
+    def evaluate(self, x, *coeffs):
+        if self.domain is not None:
+            x = poly_map_domain(x, self.domain, self.window)
+        return self.clenshaw(x, coeffs)
 
     def fit_deriv(self, x, *params):
         """
@@ -486,10 +469,11 @@ class Legendre1D(PolynomialModel):
 
         x = np.array(x, dtype=np.float, copy=False, ndmin=1)
         v = np.empty((self.degree + 1,) + x.shape, dtype=x.dtype)
-        v[0] = x * 0 + 1
-        v[1] = x
-        for i in range(2, self.degree + 1):
-            v[i] = (v[i - 1] * x * (2 * i - 1) - v[i - 2] * (i - 1)) / i
+        v[0] = 1
+        if self.degree > 0:
+            v[1] = x
+            for i in range(2, self.degree + 1):
+                v[i] = (v[i - 1] * x * (2 * i - 1) - v[i - 2] * (i - 1)) / i
         return np.rollaxis(v, 0, v.ndim)
 
     @staticmethod
@@ -546,15 +530,12 @@ class Polynomial1D(PolynomialModel):
                 super(Polynomial1D, self).prepare_inputs(x, **kwargs)
 
         x = inputs[0]
-
-        if self.domain is not None:
-            x = poly_map_domain(x, self.domain, self.window)
-
         return (x,), format_info
 
-    @classmethod
-    def evaluate(cls, x, *coeffs):
-        return cls.horner(x, coeffs)
+    def evaluate(self, x, *coeffs):
+        if self.domain is not None:
+            x = poly_map_domain(x, self.domain, self.window)
+        return self.horner(x, coeffs)
 
     def fit_deriv(self, x, *params):
         """
@@ -574,10 +555,11 @@ class Polynomial1D(PolynomialModel):
         """
 
         v = np.empty((self.degree + 1,) + x.shape, dtype=np.float)
-        v[0] = x * 0 + 1
-        v[1] = x
-        for i in range(2, self.degree + 1):
-            v[i] = v[i - 1] * x
+        v[0] = 1
+        if self.degree > 0:
+            v[1] = x
+            for i in range(2, self.degree + 1):
+                v[i] = v[i - 1] * x
         return np.rollaxis(v, 0, v.ndim)
 
     @staticmethod
@@ -640,17 +622,27 @@ class Polynomial2D(PolynomialModel):
 
         if x.shape != y.shape:
             raise ValueError("Expected input arrays to have the same shape")
+        return (x, y), format_info
 
+    def evaluate(self, x, y, *coeffs):
         if self.x_domain is not None:
             x = poly_map_domain(x, self.x_domain, self.x_window)
         if self.y_domain is not None:
             y = poly_map_domain(y, self.y_domain, self.y_window)
-
-        return (x, y), format_info
-
-    def evaluate(self, x, y, *coeffs):
         invcoeff = self.invlex_coeff(coeffs)
-        return self.multivariate_horner(x, y, invcoeff)
+        result = self.multivariate_horner(x, y, invcoeff)
+
+        # Special case for degree==0 to ensure that the shape of the output is
+        # still as expected by the broadcasting rules, even though the x and y
+        # inputs are not used in the evaluation
+        if self.degree == 0:
+            output_shape = check_broadcast(np.shape(coeffs[0]), x.shape)
+            if output_shape:
+                new_result = np.empty(output_shape)
+                new_result[:] = result
+                result = new_result
+
+        return result
 
     def fit_deriv(self, x, y, *params):
         """
@@ -702,19 +694,19 @@ class Polynomial2D(PolynomialModel):
                     name = 'c{0}_{1}'.format(j, i)
                     coeff = coeffs[self.param_names.index(name)]
                     invlex_coeffs.append(coeff)
-        return np.array(invlex_coeffs[::-1])
+        return invlex_coeffs[::-1]
 
     def multivariate_horner(self, x, y, coeffs):
         """
         Multivariate Horner's scheme
 
         Parameters
-        --------------
+        ----------
         x, y : array
         coeff : array of coefficients in inverse lexical order
         """
 
-        alpha = np.array(self._invlex())
+        alpha = self._invlex()
         r0 = coeffs[0]
         r1 = r0 * 0.0
         r2 = r0 * 0.0
@@ -968,9 +960,6 @@ class _SIP1D(PolynomialBase):
         self.coeff_prefix = coeff_prefix
         self._param_names = self._generate_coeff_names(coeff_prefix)
 
-        if params:
-            self._validate_params(**params)
-
         super(_SIP1D, self).__init__(n_models=n_models,
                                      model_set_axis=model_set_axis,
                                      name=name, meta=meta, **params)
