@@ -13,10 +13,11 @@ from numpy.testing.utils import (assert_allclose, assert_array_equal,
 from ...tests.helper import pytest
 
 from ..core import Model, ModelDefinitionError
+from ..parameters import Parameter
 from ..models import (Const1D, Shift, Scale, Rotation2D, Gaussian1D,
                       Gaussian2D, Polynomial1D, Polynomial2D,
-                      AffineTransformation2D,
-                      Identity, Mapping)
+                      Chebyshev2D, Legendre2D, Chebyshev1D, Legendre1D,
+                      AffineTransformation2D, Identity, Mapping)
 
 
 @pytest.mark.parametrize(('expr', 'result'),
@@ -714,3 +715,122 @@ def test_invalid_operands():
 
     with pytest.raises(ModelDefinitionError):
         Rotation2D(90) + Gaussian1D(1, 0, 0.1)
+
+
+class _ConstraintsTestA(Model):
+    stddev = Parameter(default=0, min=0, max=0.3)
+    mean = Parameter(default=0, fixed=True)
+
+    @staticmethod
+    def evaluate(stddev, mean):
+        return stddev, mean
+
+
+class _ConstraintsTestB(Model):
+    mean = Parameter(default=0, fixed=True)
+
+    @staticmethod
+    def evaluate(mean):
+        return mean
+
+
+@pytest.mark.parametrize('model',
+        [Gaussian1D(bounds={'stddev': (0, 0.3)}, fixed={'mean': True}) +
+            Gaussian1D(fixed={'mean': True}),
+         (_ConstraintsTestA + _ConstraintsTestB)()])
+def test_inherit_constraints(model):
+    """
+    Various tests for copying of constraint values between compound models and
+    their members.
+
+    There are two versions of this test: One where a compound model is created
+    from two model instances, and another where a compound model is created
+    from two model classes that have default constraints set on some of their
+    parameters.
+
+    Regression test for https://github.com/astropy/astropy/issues/3481
+    """
+
+    # Lots of assertions in this test as there are multiple interfaces to
+    # parameter constraints
+
+    assert 'stddev_0' in model.bounds
+    assert model.bounds['stddev_0'] == (0, 0.3)
+    assert model.stddev_0.bounds == (0, 0.3)
+    assert 'mean_0' in model.fixed
+    assert model.fixed['mean_0'] is True
+    assert model.mean_0.fixed is True
+    assert 'mean_1' in model.fixed
+    assert model.fixed['mean_1'] is True
+    assert model.mean_1.fixed is True
+
+    # Great, all the constraints were inherited properly
+    # Now what about if we update them through the sub-models?
+    model[0].stddev.bounds = (0, 0.4)
+    assert model.bounds['stddev_0'] == (0, 0.4)
+    assert model.stddev_0.bounds == (0, 0.4)
+    assert model[0].stddev.bounds == (0, 0.4)
+    assert model[0].bounds['stddev'] == (0, 0.4)
+
+    model[0].bounds['stddev'] = (0.1, 0.5)
+    assert model.bounds['stddev_0'] == (0.1, 0.5)
+    assert model.stddev_0.bounds == (0.1, 0.5)
+    assert model[0].stddev.bounds == (0.1, 0.5)
+    assert model[0].bounds['stddev'] == (0.1, 0.5)
+
+    model[1].mean.fixed = False
+    assert model.fixed['mean_1'] is False
+    assert model.mean_1.fixed is False
+    assert model[1].mean.fixed is False
+    assert model[1].fixed['mean'] is False
+
+    model[1].fixed['mean'] = True
+    assert model.fixed['mean_1'] is True
+    assert model.mean_1.fixed is True
+    assert model[1].mean.fixed is True
+    assert model[1].fixed['mean'] is True
+
+
+def test_compound_custom_inverse():
+    """
+    Test that a compound model with a custom inverse has that inverse applied
+    when the inverse of another model, of which it is a component, is computed.
+    Regression test for https://github.com/astropy/astropy/issues/3542
+    """
+
+    poly = Polynomial1D(1, c0=1, c1=2)
+    scale = Scale(1)
+    shift = Shift(1)
+
+    model1 = poly | scale
+    model1.inverse = poly
+
+    # model1 now has a custom inverse (the polynomial itself, ignoring the
+    # trivial scale factor)
+    model2 = shift | model1
+
+    assert_allclose(model2.inverse(1), (poly | shift.inverse)(1))
+
+    # Make sure an inverse is not allowed if the models were combined with the
+    # wrong operator, or if one of the models doesn't have an inverse defined
+    with pytest.raises(NotImplementedError):
+        (shift + model1).inverse
+
+    with pytest.raises(NotImplementedError):
+        (model1 & poly).inverse
+
+
+@pytest.mark.parametrize('poly', [Chebyshev2D(1, 2), Polynomial2D(2), Legendre2D(1, 2),
+                                  Chebyshev1D(5), Legendre1D(5), Polynomial1D(5)])
+def test_compound_with_polynomials(poly):
+    """
+    Tests that polynomials are scaled when used in compound models.
+    Issue #3699
+    """
+    poly.parameters = [1, 2, 3, 4, 1, 2]
+    shift = Shift(3)
+    model = poly | shift
+    x, y = np.mgrid[:20, :37]
+    result_compound = model(x, y)
+    result = shift(poly(x, y))
+    assert_allclose(result, result_compound)
