@@ -1,6 +1,6 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """
-This module prvoides the tools used to internally run the astropy test suite
+This module provides the tools used to internally run the astropy test suite
 from the installed astropy.  It makes use of the `pytest` testing framework.
 """
 from __future__ import (absolute_import, division, print_function,
@@ -17,7 +17,6 @@ import zlib
 import functools
 import multiprocessing
 import os
-import subprocess
 import shutil
 import tempfile
 import types
@@ -31,10 +30,10 @@ try:
 except ImportError:
     pass
 
-from distutils.core import Command
-
 from .. import test
-from ..utils.exceptions import AstropyWarning
+from ..utils.exceptions import (AstropyWarning,
+                                AstropyDeprecationWarning,
+                                AstropyPendingDeprecationWarning)
 from ..config import configuration
 
 if os.environ.get('ASTROPY_USE_SYSTEM_PYTEST') or '_pytest' in sys.modules:
@@ -60,12 +59,6 @@ else:
     importer = extern_pytest.DictImporter(unpacked_sources)
     sys.meta_path.insert(0, importer)
 
-    # On Python 3.1, we need to forcibly import the py.test-"bundled"
-    # argparse before importing py.test, since it isn't in the
-    # standard library, and py.test's workaround doesn't appear to
-    # work with the "absolute imports" of Python 3.x.
-    if six.PY3 and sys.version_info[1] <= 1:
-        argparse = importer.load_module(str('argparse'))
     pytest = importer.load_module(str('pytest'))
 
 
@@ -110,17 +103,10 @@ class TestRunner(object):
     def run_tests(self, package=None, test_path=None, args=None, plugins=None,
                   verbose=False, pastebin=None, remote_data=False, pep8=False,
                   pdb=False, coverage=False, open_files=False, parallel=0,
-                  docs_path=None, skip_docs=False):
+                  docs_path=None, skip_docs=False, repeat=None):
         """
         The docstring for this method lives in astropy/__init__.py:test
         """
-        try:
-            get_ipython()
-        except NameError:
-            pass
-        else:
-            raise RuntimeError(
-                "Running astropy tests inside of IPython is not supported.")
 
         if coverage:
             warnings.warn(
@@ -152,24 +138,34 @@ class TestRunner(object):
 
         if test_path:
             base, ext = os.path.splitext(test_path)
-            if ext == '.py':
-                test_path = os.path.abspath(test_path)
-                all_args.append(test_path)
-            elif ext == '.rst':
+
+            if ext in ('.rst', ''):
                 if docs_path is None:
                     # This shouldn't happen from "python setup.py test"
                     raise ValueError(
-                        "Can not test .rst files without a docs_path specified.")
-                else:
+                        "Can not test .rst files without a docs_path "
+                        "specified.")
+
+                abs_docs_path = os.path.abspath(docs_path)
+                abs_test_path = os.path.abspath(
+                    os.path.join(abs_docs_path, os.pardir, test_path))
+
+                common = os.path.commonprefix((abs_docs_path, abs_test_path))
+
+                if os.path.exists(abs_test_path) and common == abs_docs_path:
                     # Since we aren't testing any Python files within
                     # the astropy tree, we need to forcibly load the
                     # astropy py.test plugins, and then turn on the
                     # doctest_rst plugin.
-                    all_args.extend(['-p', 'astropy.tests.pytest_plugins', '--doctest-rst'])
-                    test_path = os.path.join(docs_path, '..', test_path)
-                    all_args.append(test_path)
-            else:
-                raise ValueError("Test file path must be to a .py or .rst file")
+                    all_args.extend(['-p', 'astropy.tests.pytest_plugins',
+                                     '--doctest-rst'])
+                    test_path = abs_test_path
+
+            if not (os.path.isdir(test_path) or ext in ('.py', '.rst')):
+                raise ValueError("Test path must be a directory or a path to "
+                                 "a .py or .rst file")
+
+            all_args.append(test_path)
         else:
             all_args.append(package_path)
             if docs_path is not None and not skip_docs:
@@ -210,15 +206,17 @@ class TestRunner(object):
 
         # check for opened files after each test
         if open_files:
-            try:
-                subproc = subprocess.Popen(
-                    ['lsof -F0 -n -p {0}'.format(os.getpid())],
-                    shell=True, stdout=subprocess.PIPE)
-                output = subproc.communicate()[0].strip()
-            except subprocess.CalledProcessError:
+            if parallel != 0:
                 raise SystemError(
-                    "open file detection requested, but could not "
-                    "successfully run the 'lsof' command")
+                    "open file detection may not be used in conjunction with "
+                    "parallel testing.")
+
+            try:
+                import psutil
+            except ImportError:
+                raise SystemError(
+                    "open file detection requested, but psutil package "
+                    "is not installed.")
 
             all_args.append('--open-files')
 
@@ -242,6 +240,9 @@ class TestRunner(object):
                 parallel = multiprocessing.cpu_count()
             all_args.extend(['-n', six.text_type(parallel)])
 
+        if repeat:
+            all_args.append('--repeat={0}'.format(repeat))
+
         if six.PY2:
             all_args = [x.encode('utf-8') for x in all_args]
 
@@ -366,15 +367,19 @@ class raises(object):
 
 
 _deprecations_as_exceptions = False
+_include_astropy_deprecations = True
 
-
-def enable_deprecations_as_exceptions():
+def enable_deprecations_as_exceptions(include_astropy_deprecations=True):
     """
     Turn on the feature that turns deprecations into exceptions.
     """
+
     global _deprecations_as_exceptions
     _deprecations_as_exceptions = True
 
+    global _include_astropy_deprecations
+    _include_astropy_deprecations = include_astropy_deprecations
+
 
 def treat_deprecations_as_exceptions():
     """
@@ -422,6 +427,11 @@ def treat_deprecations_as_exceptions():
     # Now, turn DeprecationWarnings into exceptions
     warnings.filterwarnings("error", ".*", DeprecationWarning)
 
+    # Only turn astropy deprecation warnings into exceptions if requested
+    if _include_astropy_deprecations:
+        warnings.filterwarnings("error", ".*", AstropyDeprecationWarning)
+        warnings.filterwarnings("error", ".*", AstropyPendingDeprecationWarning)
+
     if sys.version_info[:2] == (2, 6):
         # py.test's warning.showwarning does not include the line argument
         # on Python 2.6, so we need to explicitly ignore this warning.
@@ -439,6 +449,18 @@ def treat_deprecations_as_exceptions():
             r"'U' mode is deprecated",
             DeprecationWarning)
 
+        # BeautifulSoup4 triggers a DeprecationWarning in stdlib's
+        # html module.x
+        warnings.filterwarnings(
+            "always",
+            "The strict argument and mode are deprecated.",
+            DeprecationWarning)
+        warnings.filterwarnings(
+            "always",
+            "The value of convert_charrefs will become True in 3.5. "
+            "You are encouraged to set the value explicitly.",
+            DeprecationWarning)
+
 
 class catch_warnings(warnings.catch_warnings):
     """
@@ -537,190 +559,110 @@ def assert_follows_unicode_guidelines(
             assert eval(repr_x, roundtrip) == x
 
 
-##############################################################################
-# Note: the following class exists only for backward-compatibility purposes. #
-#       It has been moved to the separate astropy-helpers package, located   #
-#       at https://github.com/astropy/astropy-helpers. Any new development   #
-#       or bug fixes should be done there                                    #
-##############################################################################
+@pytest.fixture(params=[0, 1, -1])
+def pickle_protocol(request):
+    """
+    Fixture to run all the tests for protocols 0 and 1, and -1 (most advanced).
+    (Originally from astropy.table.tests.test_pickle)
+    """
+    return request.param
 
 
-class astropy_test(Command, object):
-    user_options = [
-        ('package=', 'P',
-         "The name of a specific package to test, e.g. 'io.fits' or 'utils'.  "
-         "If nothing is specified, all default tests are run."),
-        ('test-path=', 't',
-         'Specify a test location by path.  If a relative path to a '
-         '.py file, it is relative to the built package.  If a relative '
-         'path to a .rst file, it is relative to the docs directory '
-         '(see --docs-path).  May also be an absolute path.'),
-        ('verbose-results', 'V',
-         'Turn on verbose output from pytest.'),
-        ('plugins=', 'p',
-         'Plugins to enable when running pytest.'),
-        ('pastebin=', 'b',
-         "Enable pytest pastebin output. Either 'all' or 'failed'."),
-        ('args=', 'a',
-         'Additional arguments to be passed to pytest.'),
-        ('remote-data', 'R', 'Run tests that download remote data.'),
-        ('pep8', '8',
-         'Enable PEP8 checking and disable regular tests. '
-         'Requires the pytest-pep8 plugin.'),
-        ('pdb', 'd',
-         'Start the interactive Python debugger on errors.'),
-        ('coverage', 'c',
-         'Create a coverage report. Requires the coverage package.'),
-        ('open-files', 'o', 'Fail if any tests leave files open.'),
-        ('parallel=', 'j',
-         'Run the tests in parallel on the specified number of '
-         'CPUs.  If negative, all the cores on the machine will be '
-         'used.  Requires the pytest-xdist plugin.'),
-        ('docs-path=', None,
-         'The path to the documentation .rst files.  If not provided, and '
-         'the current directory contains a directory called "docs", that '
-         'will be used.'),
-        ('skip-docs', None,
-         "Don't test the documentation .rst files.")
-    ]
+def generic_recursive_equality_test(a, b, class_history):
+    """
+    Check if the attributes of a and b are equal. Then,
+    check if the attributes of the attributes are equal.
+    """
+    dict_a = a.__dict__
+    dict_b = b.__dict__
+    for key in dict_a:
+        assert key in dict_b,\
+          "Did not pickle {0}".format(key)
+        if hasattr(dict_a[key], '__eq__'):
+            eq = (dict_a[key] == dict_b[key])
+            if '__iter__' in dir(eq):
+                eq = (False not in eq)
+            assert eq, "Value of {0} changed by pickling".format(key)
 
-    user_options = _fix_user_options(user_options)
+        if hasattr(dict_a[key], '__dict__'):
+            if dict_a[key].__class__ in class_history:
+                #attempt to prevent infinite recursion
+                pass
+            else:
+                new_class_history = [dict_a[key].__class__]
+                new_class_history.extend(class_history)
+                generic_recursive_equality_test(dict_a[key],
+                                                dict_b[key],
+                                                new_class_history)
 
-    package_name = None
 
-    def initialize_options(self):
-        self.package = None
-        self.test_path = None
-        self.verbose_results = False
-        self.plugins = None
-        self.pastebin = None
-        self.args = None
-        self.remote_data = False
-        self.pep8 = False
-        self.pdb = False
-        self.coverage = False
-        self.open_files = False
-        self.parallel = 0
-        self.docs_path = None
-        self.skip_docs = False
+def check_pickling_recovery(original, protocol):
+    """
+    Try to pickle an object. If successful, make sure
+    the object's attributes survived pickling and unpickling.
+    """
+    f = pickle.dumps(original, protocol=protocol)
+    unpickled = pickle.loads(f)
+    class_history = [original.__class__]
+    generic_recursive_equality_test(original, unpickled,
+                                    class_history)
 
-    def finalize_options(self):
-        # Normally we would validate the options here, but that's handled in
-        # run_tests
-        pass
 
-    def run(self):
-        self.reinitialize_command('build', inplace=False)
-        self.run_command('build')
-        build_cmd = self.get_finalized_command('build')
-        new_path = os.path.abspath(build_cmd.build_lib)
+def assert_quantity_allclose(actual, desired, rtol=1.e-7, atol=None,
+                             **kwargs):
+    """
+    Raise an assertion if two objects are not equal up to desired tolerance.
 
-        if self.docs_path is None:
-            if os.path.exists('docs'):
-                self.docs_path = os.path.abspath('docs')
+    This is a :class:`~astropy.units.Quantity`-aware version of
+    :func:`numpy.testing.assert_allclose`.
+    """
+    import numpy as np
+    np.testing.assert_allclose(*_unquantify_allclose_arguments(actual, desired,
+                                                               rtol, atol),
+                               **kwargs)
 
-        # Copy the build to a temporary directory for the purposes of testing
-        # - this avoids creating pyc and __pycache__ directories inside the
-        # build directory
-        tmp_dir = tempfile.mkdtemp(prefix='astropy-test-')
-        testing_path = os.path.join(tmp_dir, os.path.basename(new_path))
-        shutil.copytree(new_path, testing_path)
-        shutil.copy('setup.cfg', testing_path)
 
-        cmd_pre = ''
-        cmd_post = ''
+def quantity_allclose(a, b, rtol=1.e-5, atol=None, **kwargs):
+    """
+    Returns True if two arrays are element-wise equal within a tolerance.
 
+    This is a :class:`~astropy.units.Quantity`-aware version of
+    :func:`numpy.allclose`.
+    """
+    import numpy as np
+    return np.allclose(*_unquantify_allclose_arguments(a, b, rtol, atol),
+                       **kwargs)
+
+
+def _unquantify_allclose_arguments(actual, desired, rtol, atol):
+    from .. import units as u
+
+    actual = u.Quantity(actual, subok=True, copy=False)
+
+    desired = u.Quantity(desired, subok=True, copy=False)
+    try:
+        desired = desired.to(actual.unit)
+    except u.UnitsError:
+        raise u.UnitsError("Units for 'desired' ({0}) and 'actual' ({1}) "
+                           "are not convertible"
+                           .format(desired.unit, actual.unit))
+
+    if atol is None:
+        # by default, we assume an absolute tolerance of 0
+        atol = u.Quantity(0)
+    else:
+        atol = u.Quantity(atol, subok=True, copy=False)
         try:
-            if self.coverage:
-                if self.parallel != 0:
-                    raise ValueError(
-                        "--coverage can not be used with --parallel")
+            atol = atol.to(actual.unit)
+        except u.UnitsError:
+            raise u.UnitsError("Units for 'atol' ({0}) and 'actual' ({1}) "
+                               "are not convertible"
+                               .format(atol.unit, actual.unit))
 
-                try:
-                    import coverage
-                except ImportError:
-                    raise ImportError(
-                        "--coverage requires that the coverage package is "
-                        "installed.")
+    rtol =  u.Quantity(rtol, subok=True, copy=False)
+    try:
+        rtol = rtol.to(u.dimensionless_unscaled)
+    except:
+        raise u.UnitsError("`rtol` should be dimensionless")
 
-                # Don't use get_pkg_data_filename here, because it
-                # requires importing astropy.config and thus screwing
-                # up coverage results for those packages.
-                coveragerc = os.path.join(
-                    testing_path, self.package_name, 'tests', 'coveragerc')
-
-                # We create a coveragerc that is specific to the version
-                # of Python we're running, so that we can mark branches
-                # as being specifically for Python 2 or Python 3
-                with open(coveragerc, 'rb') as fd:
-                    coveragerc_content = fd.read().decode('utf-8')
-                if six.PY3:
-                    ignore_python_version = '2'
-                elif six.PY2:
-                    ignore_python_version = '3'
-                coveragerc_content = coveragerc_content.replace(
-                    "{ignore_python_version}", ignore_python_version).replace(
-                        "{packagename}", self.package_name)
-                tmp_coveragerc = os.path.join(tmp_dir, 'coveragerc')
-                with open(tmp_coveragerc, 'wb') as tmp:
-                    tmp.write(coveragerc_content.encode('utf-8'))
-
-                cmd_pre = (
-                    'import coverage; '
-                    'cov = coverage.coverage(data_file="{0}", config_file="{1}"); '
-                    'cov.start();'.format(
-                        os.path.abspath(".coverage"), tmp_coveragerc))
-                cmd_post = (
-                    'cov.stop(); '
-                    'from astropy.tests.helper import _save_coverage; '
-                    '_save_coverage(cov, result, "{0}", "{1}");'.format(
-                        os.path.abspath('.'), testing_path))
-
-            if six.PY3:
-                set_flag = "import builtins; builtins._ASTROPY_TEST_ = True"
-            elif six.PY2:
-                set_flag = "import __builtin__; __builtin__._ASTROPY_TEST_ = True"
-
-            cmd = ('{cmd_pre}{0}; import {1.package_name}, sys; result = ('
-                   '{1.package_name}.test('
-                   'package={1.package!r}, '
-                   'test_path={1.test_path!r}, '
-                   'args={1.args!r}, '
-                   'plugins={1.plugins!r}, '
-                   'verbose={1.verbose_results!r}, '
-                   'pastebin={1.pastebin!r}, '
-                   'remote_data={1.remote_data!r}, '
-                   'pep8={1.pep8!r}, '
-                   'pdb={1.pdb!r}, '
-                   'open_files={1.open_files!r}, '
-                   'parallel={1.parallel!r}, '
-                   'docs_path={1.docs_path!r}, '
-                   'skip_docs={1.skip_docs!r})); '
-                   '{cmd_post}'
-                   'sys.exit(result)')
-            cmd = cmd.format(set_flag, self, cmd_pre=cmd_pre, cmd_post=cmd_post)
-
-            # Run the tests in a subprocess--this is necessary since
-            # new extension modules may have appeared, and this is the
-            # easiest way to set up a new environment
-
-            # On Python 3.x prior to 3.3, the creation of .pyc files
-            # is not atomic.  py.test jumps through some hoops to make
-            # this work by parsing import statements and carefully
-            # importing files atomically.  However, it can't detect
-            # when __import__ is used, so its carefulness still fails.
-            # The solution here (admittedly a bit of a hack), is to
-            # turn off the generation of .pyc files altogether by
-            # passing the `-B` switch to `python`.  This does mean
-            # that each core will have to compile .py file to bytecode
-            # itself, rather than getting lucky and borrowing the work
-            # already done by another core.  Compilation is an
-            # insignificant fraction of total testing time, though, so
-            # it's probably not worth worrying about.
-            retcode = subprocess.call([sys.executable, '-B', '-c', cmd],
-                                      cwd=testing_path, close_fds=False)
-        finally:
-            # Remove temporary directory
-            shutil.rmtree(tmp_dir)
-
-        raise SystemExit(retcode)
+    return actual.value, desired.value, rtol.value, atol.value
