@@ -18,6 +18,7 @@ import numpy as np
 
 from .. import units as u
 from .. import _erfa as erfa
+from ..units import UnitConversionError
 from ..utils.compat.misc import override__dir__
 from ..extern import six
 
@@ -272,6 +273,8 @@ class Time(object):
             try:
                 return FormatClass(val, val2, scale, self.precision,
                                    self.in_subfmt, self.out_subfmt)
+            except UnitConversionError:
+                raise
             except (ValueError, TypeError):
                 pass
         else:
@@ -416,6 +419,13 @@ class Time(object):
     def size(self):
         return self._time.jd1.size
 
+    def __bool__(self):
+        """Any time should evaluate to True, except when it is empty."""
+        return self.size > 0
+
+    # In python2, __bool__ is not defined.
+    __nonzero__ = __bool__
+
     @property
     def isscalar(self):
         return self.shape == ()
@@ -451,7 +461,7 @@ class Time(object):
             ``'mean'`` or ``'apparent'``, i.e., accounting for precession
             only, or also for nutation.
         longitude : `~astropy.units.Quantity`, `str`, or `None`; optional
-           The longitude on the Earth at which to compute the sidereal time.
+            The longitude on the Earth at which to compute the sidereal time.
             Can be given as a `~astropy.units.Quantity` with angular units
             (or an `~astropy.coordinates.Angle` or
             `~astropy.coordinates.Longitude`), or as a name of an
@@ -865,6 +875,9 @@ class Time(object):
     """TDB - TT time scale offset"""
 
     def __len__(self):
+        if self.isscalar:
+            raise TypeError("Scalar {0} object has no len()"
+                            .format(self.__class__.__name__))
         return len(self.jd1)
 
     def __sub__(self, other):
@@ -1319,15 +1332,14 @@ class TimeFormat(object):
             raise TypeError('Input values for {0} class must be finite doubles'
                             .format(self.name))
 
-        if hasattr(val1, 'to'):
+        if getattr(val1, 'unit', None) is not None:
             # set possibly scaled unit any quantities should be converted to
             _unit = u.CompositeUnit(getattr(self, 'unit', 1.), [u.day], [1])
             val1 = val1.to(_unit).value
             if val2 is not None:
                 val2 = val2.to(_unit).value
-        else:
-            if hasattr(val2, 'to'):
-                raise TypeError('Cannot mix float and Quantity inputs')
+        elif getattr(val2, 'unit', None) is not None:
+            raise TypeError('Cannot mix float and Quantity inputs')
 
         if val2 is None:
             val2 = np.zeros_like(val1)
