@@ -157,7 +157,11 @@ class TestBasic():
         assert np.all(t7[:, 2]._time.jd2 == t7._time.jd2[:, 2])
         assert np.all(t7[:, 0]._time.jd1 == t._time.jd1)
         assert np.all(t7[:, 0]._time.jd2 == t._time.jd2)
-
+        # Finally check empty array.
+        t8 = t[:0]
+        assert t8.isscalar is False
+        assert t8.shape == (0,)
+        assert t8.size == 0
 
     def test_properties(self):
         """Use properties to convert scales and formats.  Note that the UT1 to
@@ -389,28 +393,28 @@ class TestBasic():
     def test_utc_leap_sec(self):
         """Time behaves properly near or in UTC leap second.  This
         uses the 2012-06-30 leap second for testing."""
+        for year in ('2012', '2015'):
+            # Start with a day without a leap second and note rollover
+            t1 = Time(year + '-06-01 23:59:60.0', scale='utc')
+            assert t1.iso == year + '-06-02 00:00:00.000'
 
-        # Start with a day without a leap second and note rollover
-        t1 = Time('2012-06-01 23:59:60.0', scale='utc')
-        assert t1.iso == '2012-06-02 00:00:00.000'
+            # Leap second is different
+            t1 = Time(year + '-06-30 23:59:59.900', scale='utc')
+            assert t1.iso == year + '-06-30 23:59:59.900'
 
-        # Leap second is different
-        t1 = Time('2012-06-30 23:59:59.900', scale='utc')
-        assert t1.iso == '2012-06-30 23:59:59.900'
+            t1 = Time(year + '-06-30 23:59:60.000', scale='utc')
+            assert t1.iso == year + '-06-30 23:59:60.000'
 
-        t1 = Time('2012-06-30 23:59:60.000', scale='utc')
-        assert t1.iso == '2012-06-30 23:59:60.000'
+            t1 = Time(year + '-06-30 23:59:60.999', scale='utc')
+            assert t1.iso == year + '-06-30 23:59:60.999'
 
-        t1 = Time('2012-06-30 23:59:60.999', scale='utc')
-        assert t1.iso == '2012-06-30 23:59:60.999'
+            t1 = Time(year + '-06-30 23:59:61.0', scale='utc')
+            assert t1.iso == year + '-07-01 00:00:00.000'
 
-        t1 = Time('2012-06-30 23:59:61.0', scale='utc')
-        assert t1.iso == '2012-07-01 00:00:00.000'
-
-        # Delta time gives 2 seconds here as expected
-        t0 = Time('2012-06-30 23:59:59', scale='utc')
-        t1 = Time('2012-07-01 00:00:00', scale='utc')
-        assert allclose_sec((t1 - t0).sec, 2.0)
+            # Delta time gives 2 seconds here as expected
+            t0 = Time(year + '-06-30 23:59:59', scale='utc')
+            t1 = Time(year + '-07-01 00:00:00', scale='utc')
+            assert allclose_sec((t1 - t0).sec, 2.0)
 
     def test_init_from_time_objects(self):
         """Initialize from one or more Time objects"""
@@ -746,6 +750,34 @@ def test_dir():
     assert 'utc' in dir(t)
 
 
+def test_bool():
+    """Any Time object should evaluate to True unless it is empty [#3520]."""
+    t = Time(np.arange(50000, 50010), format='mjd', scale='utc')
+    assert bool(t) is True
+    assert bool(t[0]) is True
+    assert bool(t[:0]) is False
+
+
+def test_len_size():
+    """Check length of Time objects and that scalar ones do not have one."""
+    t = Time(np.arange(50000, 50010), format='mjd', scale='utc')
+    assert len(t) == 10 and t.size == 10
+    t1 = Time(np.arange(50000, 50010).reshape(2, 5), format='mjd', scale='utc')
+    assert len(t1) == 2 and t1.size == 10
+    # Can have length 1 or length 0 arrays.
+    t2 = t[:1]
+    assert len(t2) == 1 and t2.size == 1
+    t3 = t[:0]
+    assert len(t3) == 0 and t3.size == 0
+    # But cannot get length from scalar.
+    t4 = t[0]
+    with pytest.raises(TypeError) as err:
+        len(t4)
+    # Ensure we're not just getting the old error of
+    # "object of type 'float' has no len()".
+    assert 'Time' in str(err)
+
+
 def test_TimeFormat_scale():
     """guard against recurrence of #1122, where TimeFormat class looses uses
     attributes (delta_ut1_utc here), preventing conversion to unix, cxc"""
