@@ -7,11 +7,45 @@ Handles a "generic" string format for units
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
+from ...extern import six
+
 import os
 import re
+import warnings
 
 from . import utils
 from .base import Base
+from ...utils.misc import did_you_mean
+
+
+def _to_string(cls, unit):
+    from .. import core
+
+    if isinstance(unit, core.CompositeUnit):
+        parts = []
+
+        if cls._show_scale and unit.scale != 1:
+            parts.append('{0:g}'.format(unit.scale))
+
+        if len(unit.bases):
+            positives, negatives = utils.get_grouped_by_powers(
+                unit.bases, unit.powers)
+            if len(positives):
+                parts.append(cls._format_unit_list(positives))
+            elif len(parts) == 0:
+                parts.append('1')
+
+            if len(negatives):
+                parts.append('/')
+                unit_list = cls._format_unit_list(negatives)
+                if len(negatives) == 1:
+                    parts.append('{0}'.format(unit_list))
+                else:
+                    parts.append('({0})'.format(unit_list))
+
+        return ' '.join(parts)
+    elif isinstance(unit, core.NamedUnit):
+        return cls._get_unit_name(unit)
 
 
 class Generic(Base):
@@ -54,7 +88,7 @@ class Generic(Base):
             'CARET',
             'OPEN_PAREN',
             'CLOSE_PAREN',
-            'SQRT',
+            'FUNCNAME',
             'UNIT',
             'SIGN',
             'UINT',
@@ -95,12 +129,12 @@ class Generic(Base):
 
         # This needs to be a function so we can force it to happen
         # before t_UNIT
-        def t_SQRT(t):
-            r'sqrt'
+        def t_FUNCNAME(t):
+            r'(sqrt)|(ln)|(exp)|(log)'
             return t
 
         def t_UNIT(t):
-            r'%|[a-zA-Z][a-zA-Z_]*'
+            r"%|([YZEPTGMkhdcmunpfazy]?'((?!\d)\w)+')|((?!\d)\w)+"
             t.value = cls._get_unit(t)
             return t
 
@@ -113,10 +147,12 @@ class Generic(Base):
 
         try:
             from . import generic_lextab
-            lexer = lex.lex(optimize=True, lextab=generic_lextab)
+            lexer = lex.lex(optimize=True, lextab=generic_lextab,
+                            reflags=re.UNICODE)
         except ImportError:
             lexer = lex.lex(optimize=True, lextab='generic_lextab',
-                            outputdir=os.path.dirname(__file__))
+                            outputdir=os.path.dirname(__file__),
+                            reflags=re.UNICODE)
 
         def p_main(p):
             '''
@@ -136,10 +172,14 @@ class Generic(Base):
 
         def p_division_product_of_units(p):
             '''
-            division_product_of_units : product_of_units division unit_expression
+            division_product_of_units : division_product_of_units division product_of_units
+                                      | product_of_units
             '''
             from ..core import Unit
-            p[0] = Unit(p[1] / p[3])
+            if len(p) == 4:
+                p[0] = Unit(p[1] / p[3])
+            else:
+                p[0] = p[1]
 
         def p_inverse_unit(p):
             '''
@@ -296,7 +336,7 @@ class Generic(Base):
 
         def p_function_name(p):
             '''
-            function_name : SQRT
+            function_name : FUNCNAME
             '''
             p[0] = p[1]
 
@@ -308,7 +348,7 @@ class Generic(Base):
                 p[0] = p[3] ** 0.5
             else:
                 raise ValueError(
-                   '{0!r} is not a recognized function'.format(p[1]))
+                   "'{0}' is not a recognized function".format(p[1]))
 
         def p_error(p):
             raise ValueError()
@@ -327,86 +367,80 @@ class Generic(Base):
     def _get_unit(cls, t):
         try:
             return cls._parse_unit(t.value)
-        except ValueError:
+        except ValueError as e:
             raise ValueError(
-                "At col {0}, {1!r} is not a valid unit".format(
-                    t.lexpos, t.value))
+                "At col {0}, {1}".format(
+                    t.lexpos, six.text_type(e)))
 
     @classmethod
-    def _parse_unit(cls, s):
+    def _parse_unit(cls, s, detailed_exception=True):
         from ..core import get_current_unit_registry
         registry = get_current_unit_registry().registry
         if s == '%':
             return registry['percent']
         elif s in registry:
             return registry[s]
-        raise ValueError(
-            '{0} is not a valid unit'.format(s))
+
+        if detailed_exception:
+            raise ValueError(
+                '{0} is not a valid unit. {1}'.format(
+                    s, did_you_mean(s, registry)))
+        else:
+            raise ValueError()
 
     def parse(self, s, debug=False):
-        # This is a short circuit for the case where the string
-        # is just a single unit name
+        if not isinstance(s, six.text_type):
+            s = s.decode('ascii')
+
+        result = self._do_parse(s, debug=debug)
+        if s.count('/') > 1:
+            from ..core import UnitsWarning
+            warnings.warn(
+                "'{0}' contains multiple slashes, which is "
+                "discouraged by the FITS standard".format(s),
+                UnitsWarning)
+        return result
+
+    def _do_parse(self, s, debug=False):
         try:
-            return self._parse_unit(s)
+            # This is a short circuit for the case where the string
+            # is just a single unit name
+            return self._parse_unit(s, detailed_exception=False)
         except ValueError as e:
             try:
                 return self._parser.parse(s, lexer=self._lexer, debug=debug)
             except ValueError as e:
-                if str(e):
-                    raise ValueError("{0} in string {1!r}".format(
-                        str(e), s))
+                if six.text_type(e):
+                    raise
                 else:
                     raise ValueError(
-                        "Syntax error parsing unit string {0!r}".format(s))
+                        "Syntax error parsing unit '{0}'".format(s))
 
-    def _get_unit_name(self, unit):
+    @classmethod
+    def _get_unit_name(cls, unit):
         return unit.get_format_name('generic')
 
-    def _format_unit_list(self, units):
+    @classmethod
+    def _format_unit_list(cls, units):
         out = []
-        units.sort(key=lambda x: self._get_unit_name(x[0]).lower())
+        units.sort(key=lambda x: cls._get_unit_name(x[0]).lower())
 
         for base, power in units:
             if power == 1:
-                out.append(self._get_unit_name(base))
+                out.append(cls._get_unit_name(base))
             else:
                 power = utils.format_power(power)
                 if '/' in power:
                     out.append('{0}({1})'.format(
-                        self._get_unit_name(base), power))
+                        cls._get_unit_name(base), power))
                 else:
                     out.append('{0}{1}'.format(
-                        self._get_unit_name(base), power))
+                        cls._get_unit_name(base), power))
         return ' '.join(out)
 
-    def to_string(self, unit):
-        from .. import core
-
-        if isinstance(unit, core.CompositeUnit):
-            parts = []
-
-            if self._show_scale and unit.scale != 1:
-                parts.append('{0:g}'.format(unit.scale))
-
-            if len(unit.bases):
-                positives, negatives = utils.get_grouped_by_powers(
-                    unit.bases, unit.powers)
-                if len(positives):
-                    parts.append(self._format_unit_list(positives))
-                elif len(parts) == 0:
-                    parts.append('1')
-
-                if len(negatives):
-                    parts.append('/')
-                    unit_list = self._format_unit_list(negatives)
-                    if len(negatives) == 1:
-                        parts.append('{0}'.format(unit_list))
-                    else:
-                        parts.append('({0})'.format(unit_list))
-
-            return ' '.join(parts)
-        elif isinstance(unit, core.NamedUnit):
-            return self._get_unit_name(unit)
+    @classmethod
+    def to_string(cls, unit):
+        return _to_string(cls, unit)
 
 
 class Unscaled(Generic):
